<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 28437b45919f $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-11-09 11:11 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
//NOTE: only add "newConf" to entries set in $GLOBALS['configs']. All "temporary" entries should remain in main-Request-Scope
abstract class we_editor_preferences{
	private static $editor_reloaded = false;
	private static $jsCmd = null;

	/** @var array<string,array{string,string[]}> */
	const TABS = [
		'ui' => ['fa-tv', []],
		'editor' => ['fa-edit', [we_base_perm::ADMINISTRATOR, we_base_perm::WORK_TEMPLATES_CAN_SEE, we_base_perm::WORK_CSS_NEW, we_base_perm::WORK_JS_NEW]],
		'seolinks' => ['fa-search', [we_base_perm::ADMINISTRATOR]],
		'language' => ['fa-language', [we_base_perm::ADMINISTRATOR]],
		'countries' => ['fa-flag-checkered', [we_base_perm::ADMINISTRATOR]],
		'error_handling' => ['fa-bug', [we_base_perm::ADMINISTRATOR]],
//	'validation' => array('fa-check', we_base_perm::ADMINISTRATOR),
		'email' => ['fa-envelope-o', [we_base_perm::ADMINISTRATOR]],
		'recipients' => ['fa-envelope-o', [we_base_perm::WORK_FORMMAIL_SHOW]],
		'versions' => ['fa-history', [we_base_perm::ADMINISTRATOR]],
		'defaultAttribs' => ['fa-tags', [we_base_perm::ADMINISTRATOR]],
		'advanced' => ['fa-wrench', [we_base_perm::ADMINISTRATOR]],
		'system' => ['fa-cogs', [we_base_perm::ADMINISTRATOR]],
		'cache' => ['fa-hdd-o', [we_base_perm::ADMINISTRATOR]],
		'security' => ['fa-key', [we_base_perm::ADMINISTRATOR]],
		'proxy' => ['fa-bomb', [we_base_perm::ADMINISTRATOR]],
		'modules' => ['fa-cubes', [we_base_perm::ADMINISTRATOR]],
	];

	/**
	 * This function returns the HTML code of a dialog.
	 *
	 * @param          string                                  $name
	 * @param          we_html_multiIconBoxContent                                   $content
	 * @param          int                                     $expand             (optional)
	 * @param          string                                  $show_text          (optional)
	 * @param          string                                  $hide_text          (optional)
	 *
	 * @return         string
	 */
	private static function create_dialog(string $name, we_html_multiIconBoxContent $content, int $expand = -1, string $show_text = '', string $hide_text = ''): string{

		// Return HTML code of dialog
		return (new we_html_multiIconBox($content, $name))->marginLeft()->fold($expand, false, $show_text, $hide_text)->get();
	}

	private static function get_valueB(string $settingname): bool{
		return (bool) (self::get_value($settingname));
	}

	private static function get_valueI(string $settingname): int{
		return intval(self::get_value($settingname));
	}

	private static function get_valueS(string $settingname): string{
		$val = self::get_value($settingname);
		return is_array($val) ? implode(',', $val) : (string) $val;
	}

	/**
	 * This functions return either the saved option or the changed one.
	 *
	 * @param          string                                  $settingname
	 *
	 * @see            return_value()
	 *
	 * @return         array|string|int|bool
	 */
	private static function get_value(string $settingname){
		$all = explode('-', $settingname);
		$settingname = $all[0];
		switch($settingname){
			case 'seem_start_type':
				if(($_SESSION['prefs']['seem_start_type'] === 'document' || $_SESSION['prefs']['seem_start_type'] === 'object') && !$_SESSION['prefs']['seem_start_file']){
					return 'cockpit';
				}
				return $_SESSION['prefs']['seem_start_type'];

			case 'locale_locales':
				return we_base_country::getWeFrontendLanguagesForBackend();

			case 'locale_default':
				return we_base_WE::$Langs['DefaultFrontendLanguage'];

			case 'proxy_proxy':
				// Check for settings file
				if(file_exists(WE_CUSTOM_PATH . 'conf/proxysettings.inc.php')){
					include_once(WE_CUSTOM_PATH . 'conf/proxysettings.inc.php');
				}
				return defined('WE_PROXYHOST');

			case 'message_reporting':
				return (!empty($_SESSION['prefs']['message_reporting'])) ? $_SESSION['prefs']['message_reporting'] : (we_base_util::WE_MESSAGE_ERROR + we_base_util::WE_MESSAGE_WARNING + we_base_util::WE_MESSAGE_NOTICE);

			default:
				if(isset(we_base_preferencesConfig::USER[$settingname])){
					if(isset($all[1])){
						//handle subkey
						$tmp = we_unserialize($_SESSION['prefs'][$settingname] ?? (string) we_base_preferencesConfig::USER[$settingname][2]);
						return $tmp[$all[1]] ?? 0;
					}
					return ($_SESSION['prefs'][$settingname] ?? we_base_preferencesConfig::USER[$settingname][2]);
				}

				//if not found in global_config or other config - simply return '' - this should not happen - should we return something more error-specific?
				return defined($settingname) ?
					constant($settingname) :
					(we_base_preferencesConfig::GLOBAL[$settingname][2] ??
					(we_base_preferencesConfig::OTHER[$settingname][2] ??
					'')
					);
		}
	}

	private static function getYesNoSelect(string $setting, array $options = []): string{
		$select = (new we_html_select(['name' => 'newconf[' . $setting . ']', 'class' => 'weSelect'] + $options))
			->addOptions(['0' => g_l('prefs', '[no]'), '1' => g_l('prefs', '[yes]')])
			->selectOption(self::get_valueB($setting) ? '1' : '0');

		return $select->getHtml();
	}

	private static function getTrueFalseSelect(string $setting): string{
		$select = (new we_html_select(['name' => 'newconf[' . $setting . ']', 'class' => 'weSelect']))
			->addOptions(['0' => 'false', '1' => 'true'])
			->selectOption(self::get_valueB($setting) ? '1' : '0');

		return $select->getHtml();
	}

	/**
	 * This functions saves an option in the current session.
	 *
	 * @param          string                                  $settingvalue
	 * @param          string                                  $settingname
	 *
	 * @see            save_all_values
	 * @see            we_base_preferences::changeSourceCode()
	 *
	 * @return         void
	 */
	private static function remember_value(/* mixed */$settingvalue, string $settingname, string $comment = ''): void{
		$DB_WE = DB_WE::$inst;
		if(isset(we_base_preferencesConfig::USER[$settingname]) && $settingvalue === null){ //checkboxes -> unchecked - all other values are set by the form
			$settingvalue = 0;
		}

		//check for user-setting
		switch($settingname){
			case 'MEMCACHE_SERVERS':
				we_cache_db::reset((array) $settingvalue);
				we_cache_we::reset((array) $settingvalue);
			//no break;
			default:
				if(isset(we_base_preferencesConfig::USER[$settingname])){
					$_SESSION['prefs'][$settingname] = ($settingvalue === null ? 0 : $settingvalue);
				} else {
					$file = &we_base_preferences::$config_files['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;
			case 'seem_start_file'://don't do anything here
				return;
			case 'seem_start_type':
				switch($settingvalue){
					case 'document':
						$tmp = $_SESSION['prefs']['seem_start_file'] = we_base_request::int('seem_start_document');
						$_SESSION['prefs'][$settingname] = ($tmp ? $settingvalue : 'cockpit');
						break;
					case 'object':
						$tmp = $_SESSION['prefs']['seem_start_file'] = we_base_request::int('seem_start_object');
						$_SESSION['prefs'][$settingname] = ($tmp ? $settingvalue : 'cockpit');
						break;
					default:
						$_SESSION['prefs'][$settingname] = $settingvalue;
						break;
				}
				return;
			/* case 'sizeOpt':
			  if($settingvalue == 0){
			  $_SESSION['prefs']['weWidth'] = 0;
			  $_SESSION['prefs']['weHeight'] = 0;
			  $_SESSION['prefs']['sizeOpt'] = 0;
			  } else if(($settingvalue == 1) && we_base_request::int('newconf', -1, 'weWidth') <0 && we_base_request::int('newconf', -1, 'weHeight') <0){
			  $_SESSION['prefs']['sizeOpt'] = 1;
			  }

			  return;

			  case 'weWidth':
			  if($_SESSION['prefs']['sizeOpt'] == 1){
			  $generate_java_script = ($_SESSION['prefs']['weWidth'] != $settingvalue);

			  $_SESSION['prefs']['weWidth'] = $settingvalue;

			  if($generate_java_script){
			  $height = we_base_request::int('newconf', 0, "weHeight");
			  self::$jsCmd->addCmd('setNewWESize', $settingvalue, $height);
			  }
			  }
			  return;

			  case 'weHeight':
			  if($_SESSION['prefs']['sizeOpt'] == 1){
			  $_SESSION['prefs'][$settingname] = $settingvalue;
			  }
			  return;
			 */
			case 'editorFont':
				if(intval($settingvalue) == 0){
					$_SESSION['prefs']['editorFontname'] = 'none';
					$_SESSION['prefs']['editorFontsize'] = -1;
					$_SESSION['prefs']['editorFont'] = 0;
				} else if(($settingvalue == 1) && we_base_request::string('newconf', '', 'editorFontname') && we_base_request::int('newconf', 0, 'editorFontsize')){
					$_SESSION['prefs']['editorFont'] = 1;
				}

				if(!self::$editor_reloaded){
					self::$editor_reloaded = true;

					// editor font has changed - mark all editors to reload!
					self::$jsCmd->addCmd('reloadUsedEditors');
				}
				return;

			case 'editorCodecompletion':
				$_SESSION['prefs'][$settingname] = is_array($settingvalue) ? we_serialize($settingvalue, SERIALIZE_JSON) : '';
				return;
			case 'editorFontname':
			case 'editorFontsize':
				if($_SESSION['prefs']['editorFont'] == 1){
					$_SESSION['prefs'][$settingname] = $settingvalue;
				}

				return;

			case 'editorTooltipFont':
				if(intval($settingvalue) == 0){
					$_SESSION['prefs']['editorTooltipFontname'] = 'none';
					$_SESSION['prefs']['editorTooltipFontsize'] = -1;
					$_SESSION['prefs']['editorTooltipFont'] = 0;
				} else if(($settingvalue == 1) && we_base_request::string('newconf', '', 'editorTooltipFontname') && we_base_request::int('newconf', 0, 'editorTooltipFontsize')){
					$_SESSION['prefs']['editorTooltipFont'] = 1;
				}

				if(!self::$editor_reloaded){
					self::$editor_reloaded = true;

					// editor tooltip font has changed - mark all editors to reload!
					self::$jsCmd->addCmd('reloadUsedEditors');
				}

				return;

			case 'Language': //Handle both
				$_SESSION['prefs'][$settingname] = $settingvalue;

				if($settingvalue != we_base_WE::$Language){

					// complete webEdition reload: anpassen nach Wegfall der Frames
					self::$jsCmd->addCmd('reloadUsedEditors', 1);
				}

			case 'locale_locales':
				return;
			case 'locale_default':
				if(($loc = we_base_request::string('newconf', '', 'locale_locales')) && ($def = we_base_request::string('newconf', '', 'locale_default'))){
					$locales = explode(',', $loc);
					we_base_preferences::we_writeLanguageConfig($def, $locales);
					//remove langs not used
					DB_WE::$inst->query('DELETE FROM ' . LANGLINK_TABLE . ' WHERE Locale NOT IN ("' . implode('","', $locales) . '")');
				}
				return;

			case 'WE_COUNTRIES_TOP':
				$file = &we_base_preferences::$config_files['conf_global']['content'];
				$file = we_base_preferences::changeSourceCode('define', $file, $settingname, implode(',', array_keys(we_base_request::intList('newconf', [], 'countries'), 2)), true, $comment);
				return;

			case 'WE_COUNTRIES_SHOWN':
				$file = &we_base_preferences::$config_files['conf_global']['content'];
				$file = we_base_preferences::changeSourceCode('define', $file, $settingname, implode(',', array_keys(we_base_request::intList('newconf', [], 'countries'), 1)), true, $comment);
				return;
			case 'SYSTEM_WE_SESSION_TIME':
				//check, that a session lasts at least 90 seconds, due to we-pings, this is sufficient for WE - other is up to the user.
				$file = &we_base_preferences::$config_files['conf_global']['content'];
				$file = we_base_preferences::changeSourceCode('define', $file, $settingname, max($settingvalue, 90), true, $comment);
				return;

			case 'WE_SEEM':
				$file = &we_base_preferences::$config_files['conf_global']['content'];
				if(intval($settingvalue) == constant($settingname)){
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, ($settingvalue == 1 ? '0' : '1'), true, $comment);
				}
				return;

			case 'SIDEBAR_DISABLED':
				$file = &we_base_preferences::$config_files['conf_global']['content'];
				if($settingvalue != SIDEBAR_DISABLED){
					$file = we_base_preferences::changeSourceCode('define', $file, 'SIDEBAR_DISABLED', $settingvalue, true, $comment);
				}

				$sidebar_show_on_startup = we_base_request::bool('newconf', false, 'SIDEBAR_SHOW_ON_STARTUP');
				if(SIDEBAR_SHOW_ON_STARTUP != $sidebar_show_on_startup){
					$file = we_base_preferences::changeSourceCode('define', $file, 'newconf[SIDEBAR_SHOW_ON_STARTUP]', $sidebar_show_on_startup);
				}

				$sidebar_document = we_base_request::int('newconf', 0, 'SIDEBAR_DEFAULT_DOCUMENT');
				if(SIDEBAR_DEFAULT_DOCUMENT != $sidebar_document){
					$file = we_base_preferences::changeSourceCode('define', $file, 'newconf[SIDEBAR_DEFAULT_DOCUMENT]', $sidebar_document);
				}

				$sidebar_width = we_base_request::int('newconf', 0, 'SIDEBAR_DEFAULT_WIDTH');
				if(SIDEBAR_DEFAULT_WIDTH != $sidebar_width){
					$file = we_base_preferences::changeSourceCode('define', $file, 'newconf[SIDEBAR_DEFAULT_WIDTH]', $sidebar_width);
				}

				return;

			case 'DEFAULT_STATIC_EXT':
			case 'DEFAULT_DYNAMIC_EXT':
			case 'DEFAULT_HTML_EXT':
				if(constant($settingname) != $settingvalue){
					$file = &we_base_preferences::$config_files['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			//FORMMAIL RECIPIENTS
			case 'formmail_values':
				if($settingvalue){
					$recipients = explode('<##>', $settingvalue);
					if($recipients){
						foreach($recipients as $recipient){
							$single_recipient = explode('<#>', $recipient);

							if(isset($single_recipient[0]) && ($single_recipient[0] === '#')){
								if(!empty($single_recipient[1])){
									$DB_WE->query('INSERT INTO ' . RECIPIENTS_TABLE . ' (Email) VALUES("' . $DB_WE->escape($single_recipient[1]) . '")');
								}
							} else {
								if(!empty($single_recipient[1]) && !empty($single_recipient[0])){
									$DB_WE->query('UPDATE ' . RECIPIENTS_TABLE . ' SET Email="' . $DB_WE->escape($single_recipient[1]) . '" WHERE ID=' . intval($single_recipient[0]));
								}
							}
						}
					}
				}

				return;

			case 'formmail_deleted':
				if($settingvalue){
					$formmail_deleted = explode(',', $settingvalue);
					foreach($formmail_deleted as $del){
						$DB_WE->query('DELETE FROM ' . RECIPIENTS_TABLE . ' WHERE ID=' . intval($del));
					}
				}
				return;

			case 'active_integrated_modules':
				if(!we_base_preferences::userIsAllowed('active_integrated_modules')){
					break;
				}
				$futureModules = we_base_request::stringList('newconf', [], 'active_integrated_modules');
				$active = we_modules_info::getUserEnabledModules();
				$inactive = we_modules_info::getAllInactiveModules();
				$delNonEmpty = we_base_request::bool('removeNonEmptyTables');
				$updated = false;

				foreach($active as $cur){//remove tables if empty
					$cur = (string) $cur;
					if(!in_array($cur, $futureModules, true)){//remove tables if empty
						$updated = true;
						$tables = we_modules_info::getModuleField($cur, 'tables');
						if(!$tables){
							continue;
						}
						foreach($tables as $table){
							if(DB_WE::$inst->isTabExist($table) && ($delNonEmpty || fI('SELECT COUNT(1) FROM ' . $table) === 0)){
								DB_WE::$inst->delTable($table);
							}
						}
					}
				}

				foreach($inactive as $cur){//add new tables
					if(in_array($cur, $futureModules, true)){//remove tables if empty
						$cur = (string) $cur;
						$updated = true;
						$tables = we_modules_info::getModuleField($cur, 'tables');
						if(!$tables){
							continue;
						}
						foreach($tables as $table){
							we_update_updater::replayUpdateDB(stripTblPrefix($table) . '.sql');
						}
					}
				}
				we_base_preferences::$config_files['conf_active_integrated_modules']['content'] = '<?php
return [
\'' . implode("',\n'", $futureModules) . '\'
];';
				if($updated){
					we_cache_file::clean(we_cache_file::DOMAIN_TW);
				}
				return;

			case 'useproxy':
				if($settingvalue == 1){
					// Create/overwrite proxy settings file
					$host = we_base_request::string('newconf', '', 'proxyhost');
					$port = we_base_request::int('newconf', 0, 'proxyport');
					$user = we_base_request::string('newconf', '', 'proxyuser');
					$pass = str_replace('"', '', we_base_request::_(we_base_request::RAW_CHECKED, 'newconf', '', 'proxypass'));
					$pass = ($pass === we_customer_customer::NOPWD_CHANGE ? (defined('WE_PROXYPASSWORD') ? WE_PROXYPASSWORD : '') : $pass);
					we_base_preferences::setConfigContent('proxysettings', '<?php
	define(\'WE_PROXYHOST\', "' . $host . '");
	define(\'WE_PROXYPORT\', ' . $port . ');
	define(\'WE_PROXYUSER\', "' . $user . '");
	define(\'WE_PROXYPASSWORD\', "' . $pass . '");'
					);
				} else {
					// Delete proxy settings file
					we_base_file::deleteLocalFile(WE_CUSTOM_PATH . 'conf/proxysettings.inc.php');
					we_base_preferences::unsetConfig('proxysettings');
				}
				return;

			case 'proxyhost':
			case 'proxyport':
			case 'proxyuser':
			case 'proxypass':
				return;
			case 'SMTP_PASSWORD':
				if($settingvalue !== we_customer_customer::NOPWD_CHANGE){
					$file = &we_base_preferences::$config_files['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			// ADVANCED
			case 'DB_CONNECT':
				$file = &we_base_preferences::$config_files['conf_conf']['content'];
				$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue);
				return;

			case 'DB_SET_CHARSET':
				$file = &we_base_preferences::$config_files['conf_conf']['content'];

				if(!defined($settingname) || $settingvalue != constant($settingname)){
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			case 'useauth':
				$file = &we_base_preferences::$config_files['conf_conf']['content'];
				if($settingvalue == 1){
					// enable
					if(!(defined('HTTP_USERNAME')) || !(defined('HTTP_PASSWORD'))){
						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_USERNAME', 'myUsername', false);
						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_PASSWORD', 'myPassword', false);
					}

					$un = defined('HTTP_USERNAME') ? HTTP_USERNAME : '';
					$un1 = we_base_request::string('newconf', '', 'HTTP_USERNAME');
					$pw1 = we_base_request::string('newconf', '', 'HTTP_PASSWORD');
					if($un != $un1 || $pw1 != we_customer_customer::NOPWD_CHANGE){

						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_USERNAME', $un1);
						$file = ($pw1 === we_customer_customer::NOPWD_CHANGE ? $file : we_base_preferences::changeSourceCode('define', $file, 'HTTP_PASSWORD', $pw1));
					}
				} else {
					// disable
					if(defined('HTTP_USERNAME') || defined('HTTP_PASSWORD')){
						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_USERNAME', 'myUsername', false);
						$file = we_base_preferences::changeSourceCode('define', $file, 'HTTP_PASSWORD', 'myPassword', false);
					}
				}
				return;

			case 'HTTP_USERNAME':
			case 'HTTP_PASSWORD':
				return;

			//ERROR HANDLING
			case 'WE_ERROR_NOTICES':
			case 'DEBUG_SLOW_QUERY':
			case 'WE_ERROR_SHOW':
				$file = &we_base_preferences::$config_files['conf_global']['content'];

				if($settingvalue != constant($settingname)){
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;

			case 'WE_ERROR_MAIL':
				$file = &we_base_preferences::$config_files['conf_global']['content'];

				if(!$settingvalue && WE_ERROR_MAIL){
					$file = we_base_preferences::changeSourceCode('define', $file, 'WE_ERROR_MAIL', '0', true, $comment);
				} else if($settingvalue && !WE_ERROR_MAIL){
					$file = we_base_preferences::changeSourceCode('define', $file, 'WE_ERROR_MAIL', '1', true, $comment);
				}
				return;

			case 'WE_ERROR_MAIL_ADDRESS':
				$file = &we_base_preferences::$config_files['conf_global']['content'];

				if(WE_ERROR_MAIL_ADDRESS != $settingvalue){
					$file = we_base_preferences::changeSourceCode('define', $file, 'WE_ERROR_MAIL_ADDRESS', $settingvalue, true, $comment);
				}
				return;

			case 'ERROR_DOCUMENT_NO_OBJECTFILE':
				if(!defined($settingname) || constant($settingname) != $settingvalue){
					$file = &we_base_preferences::$config_files['conf_global']['content'];
					$file = we_base_preferences::changeSourceCode('define', $file, $settingname, $settingvalue, true, $comment);
				}
				return;
		}
	}

	/**
	 * This functions saves all options.
	 *
	 * @see            remember_value()
	 *
	 * @return         void
	 */
	private static function save_all_values(): void{
		we_base_preferences::loadConfigs();
		//set config to latest version
		$_REQUEST['newconf']['CONF_SAVED_VERSION'] = WE_SVNREV;
		// Second, change sourcecodes of the configfiles
		foreach(we_base_preferencesConfig::ALL_CONFIG as $name => $conf){
			foreach($conf as $key => $default){
				switch($name){
					case 'conf':
					case 'global'://no settings in session
						if(we_base_preferences::userIsAllowed($key)){
							self::remember_value(we_base_request::_($default[1], 'newconf', null, $key), $key, $default[0]);
						}
						break;
					case 'user':
						self::remember_value(we_base_request::_($default[1], 'newconf', null, $key), $key);
						break;
					default:
						if(we_base_preferences::userIsAllowed($key)){
							self::remember_value(we_base_request::_($default[1], 'newconf', null, $key), $key);
						}
						break;
				}
			}
		}
		if(!empty($_SESSION['weS']['versions']['logPrefs'])){
			$logPrefsChanged = [];
			foreach(array_keys($_SESSION['weS']['versions']['logPrefs']) as $k){
				if(isset($_REQUEST['newconf'][$k])){
					if($_SESSION['weS']['versions']['logPrefs'][$k] != $_REQUEST['newconf'][$k]){
						$logPrefsChanged[$k] = $_REQUEST['newconf'][$k];
					}
				} elseif($_SESSION['weS']['versions']['logPrefs'][$k] != ''){
					$logPrefsChanged[$k] = '';
				}
			}

			if($logPrefsChanged){
				$versionslog = new we_versions_log();
				$versionslog->saveVersionsLog($logPrefsChanged, we_versions_log::VERSIONS_PREFS);
			}
			unset($_SESSION['weS']['versions']['logPrefs']);
		}

		//SAVE CHANGES
		// Third save all changes of the config files
		we_base_preferences::saveConfigs();
	}

	private static function getAdvancedTab(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}
		$settings = new we_html_multiIconBoxContent();

		/*
		  $WYSIWYG_TYPE = new we_html_select(['name' => "newconf[WYSIWYG_TYPE]", 'class' => "weSelect"]);
		  $options = ['tinyMCE' => 'tinyMCE', 'tinyMCE_beta' => 'tinyMCE4 (Apha - not for productive use!)'];
		  foreach($options as $key => $val){
		  $WYSIWYG_TYPE->addOption($key, $val);
		  }
		  $WYSIWYG_TYPE->selectOption(self::get_valueS("WYSIWYG_TYPE"));
		  $settings = (new we_html_multiIconBoxContent())
		  ->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[wysiwyg_type]'), 'html' => $WYSIWYG_TYPE->getHtml(),
		  ->space(we_html_multiIconBox::SPACE_BIG)]);
		 */

		$we_max_upload_size = '<table class="default"><tr><td>' .
			we_html_tools::textInput("newconf[FILE_UPLOAD_MAX_UPLOAD_SIZE]", self::get_valueS("FILE_UPLOAD_MAX_UPLOAD_SIZE"), 0, [], "number", '60px') . ' MB</td><td style="padding-left:20px;" class="small">' .
			g_l('prefs', '[upload][we_max_size_hint]') .
			'</td></tr></table>';

		$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[upload][we_max_size]'), $we_max_upload_size))
				->space(we_html_multiIconBox::SPACE_BIG));

		$op = we_fileupload_ui_base::$scaleProps;
		$op[''] = g_l('siteimport', '[nolimit]');
		$img = (new we_html_select(['name' => "newconf[FILE_UPLOAD_IMG_MAX_SIZE]", 'class' => "weSelect"]))
			->addOptions($op)
			->selectOption(self::get_valueS("FILE_UPLOAD_IMG_MAX_SIZE"));
		//LANGUAGE

		$we_max_img_upload_size = $img->getHtml() . ' px ';
		//	Look which languages are installed ...
		$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[upload][we_max_img_size]'), $we_max_img_upload_size))
				->space(we_html_multiIconBox::SPACE_BIG));



		/* $WYSIWYG_TYPE_FRONTEND = new we_html_select(array('name' => "newconf[WYSIWYG_TYPE_FRONTEND]", 'class' => "weSelect"));
		  $options = array('tinyMCE' => 'tinyMCE', 'default' => 'webEdition Editor (deprecated))');
		  foreach($options as $key => $val){
		  $WYSIWYG_TYPE_FRONTEND->addOption($key, $val);
		  }
		  $WYSIWYG_TYPE_FRONTEND->selectOption(self::get_value("WYSIWYG_TYPE_FRONTEND"));
		  $settings[] = array('headline' => "Editor für textareas im Frontend", 'html' => $WYSIWYG_TYPE_FRONTEND->getHtml(),
		  ->space(we_html_multiIconBox::SPACE_BIG));
		 */
		$we_doctype_workspace_behavior = self::get_valueB("WE_DOCTYPE_WORKSPACE_BEHAVIOR");
		$we_doctype_workspace_behavior_table = '<table class="default"><tr><td>' .
			we_html_forms::radiobutton('0', (!$we_doctype_workspace_behavior), "newconf[WE_DOCTYPE_WORKSPACE_BEHAVIOR]", g_l('prefs', '[we_doctype_workspace_behavior_0]'), true, '', '', false, g_l('prefs', '[we_doctype_workspace_behavior_hint0]'), 0, 430) .
			'</td></tr><tr><td style="padding-top:10px;">' .
			we_html_forms::radiobutton('1', $we_doctype_workspace_behavior, "newconf[WE_DOCTYPE_WORKSPACE_BEHAVIOR]", g_l('prefs', '[we_doctype_workspace_behavior_1]'), true, '', '', false, g_l('prefs', '[we_doctype_workspace_behavior_hint1]'), 0, 430) .
			'</td></tr></table>';
		// Lang notice
		$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[we_doctype_workspace_behavior]'), $we_doctype_workspace_behavior_table))
				->space(we_html_multiIconBox::SPACE_BIG));
		// Build dialog
		// Build select box
		$NAVIGATION_ENTRIES_FROM_DOCUMENT = new we_html_select(['name' => "newconf[NAVIGATION_ENTRIES_FROM_DOCUMENT]", 'class' => "weSelect"]);
		for($i = 0; $i < 2; $i++){
			$NAVIGATION_ENTRIES_FROM_DOCUMENT->addOption((string) $i, g_l('prefs', $i == 0 ? '[navigation_entries_from_document_folder]' : '[navigation_entries_from_document_item]'));
		}
		$NAVIGATION_ENTRIES_FROM_DOCUMENT->selectOption(self::get_valueB("NAVIGATION_ENTRIES_FROM_DOCUMENT") ? '1' : '0');
		$settings
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[navigation_entries_from_document]'), $NAVIGATION_ENTRIES_FROM_DOCUMENT->getHtml()))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[navigation_rules_continue]'), self::getYesNoSelect('NAVIGATION_RULES_CONTINUE_AFTER_FIRST_MATCH')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[disable_modtime][title]') . we_html_tools::alertAttentionBox(g_l('prefs', '[disable_modtime][information]', false, true), we_html_tools::TYPE_HELP), self::getYesNoSelect('DISABLE_MEDIA_MODTIME')))
				->space(we_html_multiIconBox::SPACE_BIG))
		;



		return self::create_dialog("", $settings, -1);
	}

	private static function getProxyTab(): string{
		if(!we_base_preferences::userIsAllowed('useproxy')){
			return '';
		}
		/**
		 * Proxy server
		 */
		$proxy = self::get_valueB('proxy_proxy');

		$use_proxy = we_html_forms::checkbox('1', $proxy, 'newconf[useproxy]', g_l('prefs', '[useproxy]', false, true), false, '', "set_state();");
		$proxyaddr = we_html_tools::textInput('newconf[proxyhost]', self::get_valueS('WE_PROXYHOST'), 0, [], "text", '225px', false, !$proxy);
		$proxyport = we_html_tools::textInput('newconf[proxyport]', self::get_valueS('WE_PROXYPORT'), 0, [], "text", '225px', false, !$proxy);
		$proxyuser = we_html_tools::textInput('newconf[proxyuser]', self::get_valueS('WE_PROXYUSER'), 0, ['autocomplete' => 'off'], 'text', '225px', false, !$proxy);
		$proxypass = we_html_tools::textInput('newconf[proxypass]', we_customer_customer::NOPWD_CHANGE, 0, ['autocomplete' => 'off'], 'password', '225px', false, !$proxy);

		// DEFAULT CHARSET

		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('prefs', '[proxy_information]', false, true), we_html_tools::TYPE_INFO, 450, false)))->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[tab][proxy]'), $use_proxy))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[proxyaddr]'), $proxyaddr))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[proxyport]'), $proxyport))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[proxyuser]'), $proxyuser))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[proxypass]'), $proxypass))
			->space(we_html_multiIconBox::SPACE_BIG)
			->noline());
		// Build dialog element if user has permission
		return self::create_dialog("", $settings, -1);
	}

	private static function getSystemTab(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}
		//AMOUNT COLUMNS IN COCKPIT
		// Get webEdition extensions
		$we_extensions = we_base_ContentTypes::getExtension(we_base_ContentTypes::WEDOCUMENT);

		// Build static webEdition extensions select box
		$static_we_extensions = new we_html_select(['name' => 'newconf[DEFAULT_STATIC_EXT]', 'class' => 'weSelect']);
		$dynamic_we_extensions = new we_html_select(['name' => 'newconf[DEFAULT_DYNAMIC_EXT]', 'class' => 'weSelect']);
		foreach($we_extensions as $value){
			$static_we_extensions->addOption((string) $value, (string) $value);
			$dynamic_we_extensions->addOption((string) $value, (string) $value);
		}
		$static_we_extensions->selectOption(self::get_valueS('DEFAULT_STATIC_EXT'));
		$dynamic_we_extensions->selectOption(self::get_valueS('DEFAULT_DYNAMIC_EXT'));

		$we_extensions_html = g_l('prefs', '[static]') . we_html_element::br() . $static_we_extensions->getHtml() . we_html_element::br(2) . g_l('prefs', '[dynamic]') . we_html_element::br() . $dynamic_we_extensions->getHtml();

		// Build maximize window
		$html_extensions = we_base_ContentTypes::getExtension(we_base_ContentTypes::HTML);

		// Build dialog if user has permission
		$static_html_extensions = new we_html_select(['name' => 'newconf[DEFAULT_HTML_EXT]', 'class' => 'weSelect']);
		foreach($html_extensions as $value){
			$static_html_extensions->addOption((string) $value, (string) $value);
		}
		$static_html_extensions->selectOption(self::get_valueS('DEFAULT_HTML_EXT'));

		$html_extensions_html = g_l('prefs', '[html]') . '<br/>' . $static_html_extensions->getHtml();
		// SEEM start document
		// Cockpit
		$we_new_folder_mod = '<table class="default"><tr><td>' .
			we_html_tools::textInput("newconf[WE_NEW_FOLDER_MOD]", self::get_valueS("WE_NEW_FOLDER_MOD"), 3, [], "number", '60px') . '</td><td style="padding-left:20px;" class="small">' .
			g_l('prefs', '[we_new_folder_mod_hint]') .
			'</td></tr></table>';


		$db_connect = (new we_html_select(['name' => 'newconf[DB_CONNECT]', 'class' => 'weSelect']))
			->addOptions([
				'mysqli_connect' => 'mysqli_connect',
				'mysqli_pconnect' => 'mysqli_pconnect'
			])
			->selectOption(DB_CONNECT);
		// Object
		$auth = !empty(self::get_valueS("HTTP_USERNAME"));

		// Document
		$use_auth = we_html_element::hidden('newconf[useauth]', (string) $auth) .
			we_html_forms::checkbox('1', $auth, 'useauthEnabler', g_l('prefs', '[useauth]', false, true), false, '', "set_state_auth();");

		/**
		 * User name
		 */
		$authuser = we_html_tools::textInput('newconf[HTTP_USERNAME]', self::get_valueS("HTTP_USERNAME"), 0, ['autocomplete' => 'off'], "text", '225px', false, !$auth);
		$authpass = we_html_tools::textInput('newconf[HTTP_PASSWORD]', we_customer_customer::NOPWD_CHANGE, 0, ['autocomplete' => 'off'], 'password', '225px', false, !$auth);


		if(we_base_imageEdit::gd_version() > 0){ //  gd lib ist installiert
			$but = we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'newconf[WE_THUMBNAIL_DIRECTORY]', WE().consts.contentTypes.FOLDER_DOCUMENT, document.forms[0].elements['newconf[WE_THUMBNAIL_DIRECTORY]'].value, '')") : "";
			$inp = we_html_tools::textInput("newconf[WE_THUMBNAIL_DIRECTORY]", self::get_valueS("WE_THUMBNAIL_DIRECTORY"), 0, [], 'text', '125px');
			$thumbnail_dir = $inp . $but;
		} else { //  gd lib ist nicht installiert
			$but = we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? we_html_button::create_button(we_html_button::SELECT, "#", '', '', true) : "";
			$inp = we_html_tools::textInput("newconf[WE_THUMBNAIL_DIRECTORY]", self::get_valueS("WE_THUMBNAIL_DIRECTORY"), 0, [], 'text', '125px', false, true);
			$thumbnail_dir = $inp . $but . '<br/>' . g_l('thumbnails', '[add_description_nogdlib]');
		}

		$hooksHtml = self::getYesNoSelect('EXECUTE_HOOKS');
		// Build SEEM select start document chooser
		$useSession = ''; //self::getYesNoSelect('SYSTEM_WE_SESSION', ['onchange' => 'WE().util.showMessage(\'' . g_l('prefs', '[session][crypt][alert]') . '\', WE().consts.message.WE_MESSAGE_ERROR);']);
		$sessionTime = '<table class="default"><tr><td>' .
			we_html_tools::textInput("newconf[SYSTEM_WE_SESSION_TIME]", (string) (abs(self::get_valueI("SYSTEM_WE_SESSION_TIME"))), 0, [], 'number', '60px') . '</td><td style="padding-left:20px;" class="small">s</td></tr></table>';

		$cryptSession = (new we_html_select(['name' => 'newconf[SYSTEM_WE_SESSION_CRYPT]', 'class' => "weSelect", 'onchange' => 'WE().util.showMessage(\'' . g_l('prefs', '[session][crypt][alert]') . '\', WE().consts.message.WE_MESSAGE_ERROR);']))
			->addOptions([
				(string) we_base_sessionHandler::CRYPT_NONE => g_l('prefs', '[no]'),
				(string) we_base_sessionHandler::CRYPT_TRANSPARENT => 'Transparent',
				(string) we_base_sessionHandler::CRYPT_COOKIE => 'Cookie'
			])
			->selectOption(self::get_valueS('SYSTEM_WE_SESSION_CRYPT'));

		// Build SEEM select start object chooser
		$_Schedtrigger_setting = (new we_html_select(['name' => 'newconf[SCHEDULER_TRIGGER]', 'class' => "weSelect"]))
			->addOptions([
				(string) we_schedule_scheduler::TRIGGER_PREDOC => g_l('prefs', '[we_scheduler_trigger][preDoc]'), //pre
				(string) we_schedule_scheduler::TRIGGER_POSTDOC => g_l('prefs', '[we_scheduler_trigger][postDoc]'), //post
				(string) we_schedule_scheduler::TRIGGER_CRON => g_l('prefs', '[we_scheduler_trigger][cron]')
			]) //cron
			->selectOption(self::get_valueS('SCHEDULER_TRIGGER'));

		$maintenanceMode = (new we_html_select(['name' => 'newconf[MAINTENANCE_MODE]', 'class' => 'weSelect']))
			->addOptions([
				'' => '',
				we_base_constants::MAINTENANCE_FRONTEND => we_base_constants::MAINTENANCE_FRONTEND,
				we_base_constants::MAINTENANCE_BACKEND => we_base_constants::MAINTENANCE_BACKEND,
				we_base_constants::MAINTENANCE_ALL => we_base_constants::MAINTENANCE_ALL,
			])
			->selectOption(self::get_valueS('MAINTENANCE_MODE'));
		$maintenanceDoc = we_gui_suggest::getInstance()
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML])
			->setInput('newconf[MAINTENANCE_DOCUMENT]', (MAINTENANCE_DOCUMENT ?: ''))
			->setMaxResults(20)
			->setResult('MAINTENANCE_DOCUMENT_id', (string) (MAINTENANCE_DOCUMENT ? path_to_id(MAINTENANCE_DOCUMENT) : 0))
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(300)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.forms[0].elements['MAINTENANCE_DOCUMENT_id'].value,table:WE().consts.tables.FILE_TABLE, targetID:'MAINTENANCE_DOCUMENT_id',targetName:'newconf[MAINTENANCE_DOCUMENT]',filter:'" . we_base_ContentTypes::WEDOCUMENT . ',' . we_base_ContentTypes::HTML . "', open_doc:true})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['newconf[MAINTENANCE_DOCUMENT]'].value = 0;document.forms[0].elements.MAINTENANCE_DOCUMENT_id.value = ''"));

		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[maintenance][mode]'), $maintenanceMode->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED2)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[maintenance][document]'), $maintenanceDoc->getHTML()))
				->space(we_html_multiIconBox::SPACE_BIG)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[we_extensions]'), $we_extensions_html))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[html_extensions]'), $html_extensions_html))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[we_new_folder_mod]'), $we_new_folder_mod))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[db_connect]', false, true), $db_connect->getHtml()))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			/* ->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[db_set_charset]'), 'html' => $db_set_charset->getHtml() . we_html_tools::alertAttentionBox(g_l('prefs', '[db_set_charset_information]', false, true), we_html_tools::TYPE_HELP) . $html_db_charset_warning,

			  ->space(we_html_multiIconBox::SPACE_BIG)]) */
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[auth]'), $use_auth))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[authuser]'), $authuser))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[authpass]'), $authpass))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[thumbnail_dir]'), $thumbnail_dir))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[hooks]'), $hooksHtml . we_html_tools::alertAttentionBox(g_l('prefs', '[hooks_information]', false, true), we_html_tools::TYPE_HELP)))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[we_scheduler_trigger][head]'), we_modules_info::isActive(we_modules_info::SCHEDULER) ? $_Schedtrigger_setting->getHtml() . we_html_tools::alertAttentionBox(g_l('prefs', '[we_scheduler_trigger][description]', false, true), we_html_tools::TYPE_HELP) : '-'))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[session][title]'), $useSession . we_html_tools::alertAttentionBox(g_l('prefs', '[session][information]', false, true), we_html_tools::TYPE_HELP)))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[session][time]'), $sessionTime))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[session][crypt][title]'), $cryptSession->getHtml()))
			->space(we_html_multiIconBox::SPACE_BIG));
		// Build dialog element if user has permission
		return self::create_dialog('', $settings, -1);
	}

	// Build SEEM select start object chooser
	private static function getSeoDialog(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}

		$weSuggest = we_gui_suggest::getInstance();
		// Build final HTML code
		$navigation_directoryindex_names = we_html_tools::textInput("newconf[NAVIGATION_DIRECTORYINDEX_NAMES]", self::get_valueS("NAVIGATION_DIRECTORYINDEX_NAMES"), 0, [
				], 'text', '225px');

		$weSuggest
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML])
			->setInput('error_document_no_objectfile_text', ( ERROR_DOCUMENT_NO_OBJECTFILE ? id2path(ERROR_DOCUMENT_NO_OBJECTFILE) : ''))
			->setMaxResults(20)
			->setResult('newconf[ERROR_DOCUMENT_NO_OBJECTFILE]', (string) (ERROR_DOCUMENT_NO_OBJECTFILE ?: 0))
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(300)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.forms[0].elements['newconf[ERROR_DOCUMENT_NO_OBJECTFILE]'].value,table:WE().consts.tables.FILE_TABLE, targetID:'newconf[ERROR_DOCUMENT_NO_OBJECTFILE]',targetName:'error_document_no_objectfile_text',filter:'" . we_base_ContentTypes::WEDOCUMENT . ',' . we_base_ContentTypes::HTML . "', open_doc:true})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['newconf[ERROR_DOCUMENT_NO_OBJECTFILE]'].value = 0;document.forms[0].elements.error_document_no_objectfile_text.value = ''"));

		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[general_directoryindex_hide]'), we_html_tools::alertAttentionBox(g_l('prefs', '[navigation_directoryindex_description]'), we_html_tools::TYPE_INFO, 480)))
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[navigation_directoryindex_hide]'), self::getTrueFalseSelect('NAVIGATION_DIRECTORYINDEX_HIDE')))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[wysiwyglinks_directoryindex_hide]'), self::getTrueFalseSelect('WYSIWYGLINKS_DIRECTORYINDEX_HIDE')))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[navigation_directoryindex_names]'), $navigation_directoryindex_names))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('prefs', '[general_directoryindex_hide_description]'), we_html_tools::TYPE_INFO, 480)))
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[taglinks_directoryindex_hide]'), self::getTrueFalseSelect('TAGLINKS_DIRECTORYINDEX_HIDE')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[general_objectseourls]'), we_html_tools::alertAttentionBox(g_l('prefs', '[objectseourls_lowercase_description]'), we_html_tools::TYPE_HELP)))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[objectseourls_lowercase]'), self::getTrueFalseSelect('OBJECTSEOURLS_LOWERCASE')))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[navigation_objectseourls]'), self::getTrueFalseSelect('NAVIGATION_OBJECTSEOURLS')))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[wysiwyglinks_objectseourls]'), self::getTrueFalseSelect('WYSIWYGLINKS_OBJECTSEOURLS')))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(we_html_tools::alertAttentionBox(g_l('prefs', '[general_objectseourls_description]'), we_html_tools::TYPE_INFO, 480), ''))
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[taglinks_objectseourls]'), self::getTrueFalseSelect('TAGLINKS_OBJECTSEOURLS')))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[urlencode_objectseourls]'), self::getTrueFalseSelect('URLENCODE_OBJECTSEOURLS')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[general_seoinside]'), we_html_tools::alertAttentionBox(g_l('prefs', '[general_seoinside_description]'), we_html_tools::TYPE_HELP)))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[seoinside_hideineditmode]'), self::getTrueFalseSelect('SEOINSIDE_HIDEINEDITMODE')))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[seoinside_hideinwebedition]'), self::getTrueFalseSelect('SEOINSIDE_HIDEINWEBEDITION')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[error_no_object_found]'), $weSuggest->getHTML()))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[suppress404code]'), self::getTrueFalseSelect('SUPPRESS404CODE')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[force404redirect]'), self::getTrueFalseSelect('FORCE404REDIRECT') . we_html_tools::alertAttentionBox(g_l('prefs', '[force404redirect_description]'), we_html_tools::TYPE_HELP)))
			->space(we_html_multiIconBox::SPACE_BIG));
		return self::create_dialog('', $settings, -1, '', '');
	}

	private static function getErrorTab(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}

		/**
		 * Error handler
		 */
		$foldAt = 4;
		// Build dialog if user has permission

		$we_error_handler = we_html_forms::checkbox('1', true, 'newconf[WE_ERROR_HANDLER]', g_l('prefs', '[error_use_handler]', false, true), false, '', 'set_state_error_handler();', true);


		// Settings
		$error_handling_table = implode(we_html_element::br(), [
			we_html_forms::checkbox('1', true, 'newconf[WE_ERROR_ERRORS]', g_l('prefs', '[error_errors]'), false, '', '', true),
			we_html_forms::checkbox('1', true, 'newconf[WE_ERROR_WARNINGS]', g_l('prefs', '[error_warnings]'), false, '', '', true),
			we_html_forms::checkbox('1', self::get_valueB('WE_ERROR_NOTICES'), 'newconf[WE_ERROR_NOTICES]', g_l('prefs', '[error_notices]')),
			we_html_forms::checkbox('1', true, 'newconf[WE_ERROR_DEPRECATED]', g_l('prefs', '[error_deprecated]'), false, '', '', true),
			we_html_forms::checkbox('1', (bool) self::get_value('DEBUG_SLOW_QUERY'), 'newconf[DEBUG_SLOW_QUERY]', g_l('prefs', '[debug_slowQuery]'), false, '', '')
		]);

		$error_mail_specify_table = (new we_html_table(['class' => 'default defaultfont', 'style' => 'margin-left:25px;'], 1, 4))
			->setCol(0, 1, g_l('prefs', '[error_mail_address]') . ': ')
			->setCol(0, 2, we_html_tools::textInput('newconf[WE_ERROR_MAIL_ADDRESS]', self::get_valueS('WE_ERROR_MAIL_ADDRESS'), 100, [
				'placeholder' => "mail@example"], 'email', '195px'), ['style' => 'text-align:left']);

		$error_display_table = implode(we_html_element::br(), [
			we_html_forms::checkbox('1', self::get_valueB('WE_ERROR_SHOW'), 'newconf[WE_ERROR_SHOW]', g_l('prefs', '[error_display]')),
			we_html_forms::checkbox('1', true, 'newconf[WE_ERROR_LOG]', g_l('prefs', '[error_log]'), false, '', '', true),
			we_html_forms::checkbox('1', self::get_valueB('WE_ERROR_MAIL'), 'newconf[WE_ERROR_MAIL]', g_l('prefs', '[error_mail]')),
			$error_mail_specify_table->getHtml()
		]);
		// Show on Startup
		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[tab][error_handling]'), $we_error_handler))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[error_types]'), $error_handling_table))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[error_displaying]') . we_html_tools::alertAttentionBox(g_l('prefs', '[error_notices_warning]'), we_html_tools::TYPE_HELP), $error_display_table))
			->space(we_html_multiIconBox::SPACE_BIG));
		// Sidebar width
		return self::create_dialog('settings_error_expert', $settings, $foldAt, g_l('prefs', '[show_expert]'), g_l('prefs', '[hide_expert]'));
	}

	// Sidebar document
	private static function getSecurityTab(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}
		$weSuggest = we_gui_suggest::getInstance();
		$row = 0;
		$customer_table = (new we_html_table(['class' => 'default defaultfont', 'id' => 'customer_table'], 9, 10))
			->setColAttributes($row, 0, ['width' => '20px'])
			->setCol($row, 1, g_l('prefs', '[security][customer][disableLogins]') . ':', ['colspan' => '5'])
			->setColAttributes($row, 6, ['width' => '300'])
			->setCol(++$row, 1, g_l('prefs', '[security][customer][sameIP]'))
			->setColAttributes($row, 2, ['width' => '20px'])
			->setCol($row, 3, we_html_tools::textInput('newconf[SECURITY_LIMIT_CUSTOMER_IP]', self::get_valueS('SECURITY_LIMIT_CUSTOMER_IP'), 3, [
					], 'number', '50px'))
			->setCol($row, 4, '/', ['style' => 'width:2em;text-align:center'])
			->setCol($row, 5, we_html_tools::textInput('newconf[SECURITY_LIMIT_CUSTOMER_IP_HOURS]', self::get_valueS('SECURITY_LIMIT_CUSTOMER_IP_HOURS'), 3, [], 'number', '50px'))
			->setCol($row, 6, 'h')
			->setCol(++$row, 1, g_l('prefs', '[security][customer][sameUser]'))
			->setCol($row, 3, we_html_tools::textInput('newconf[SECURITY_LIMIT_CUSTOMER_NAME]', self::get_valueS('SECURITY_LIMIT_CUSTOMER_NAME'), 3, [], 'number', '50px'))
			->setCol($row, 4, '/', ['style' => 'text-align:center;'])
			->setCol($row, 5, we_html_tools::textInput('newconf[SECURITY_LIMIT_CUSTOMER_NAME_HOURS]', self::get_valueS('SECURITY_LIMIT_CUSTOMER_NAME_HOURS'), 3, [
					], 'number', '50px'))
			->setCol($row, 6, 'h')
			->setCol(++$row, 1, g_l('prefs', '[security][customer][errorPage]'));

		$weSuggest->setAcId('SECURITY_LIMIT_CUSTOMER_REDIRECT_doc')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML])
			->setInput('SECURITY_LIMIT_CUSTOMER_REDIRECT_text', (SECURITY_LIMIT_CUSTOMER_REDIRECT ? id2path(SECURITY_LIMIT_CUSTOMER_REDIRECT) : ''))
			->setMaxResults(20)
			->setResult('newconf[SECURITY_LIMIT_CUSTOMER_REDIRECT]', (string) (SECURITY_LIMIT_CUSTOMER_REDIRECT ?: 0))
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(250)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document', {id:document.forms[0].elements['newconf[SECURITY_LIMIT_CUSTOMER_REDIRECT]'].value,table:WE().consts.tables.FILE_TABLE, targetID:'newconf[SECURITY_LIMIT_CUSTOMER_REDIRECT]',targetName:'SECURITY_LIMIT_CUSTOMER_REDIRECT_text',filter:'" . we_base_ContentTypes::WEDOCUMENT . "," . we_base_ContentTypes::HTML . "', open_doc:true})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['newconf[SECURITY_LIMIT_CUSTOMER_REDIRECT]'].value = 0;document.forms[0].elements.SECURITY_LIMIT_CUSTOMER_REDIRECT_text.value = ''"));
		// build html
		$customer_table
			->setCol($row, 3, $weSuggest->getHTML(), ['colspan' => '5'])
			->setCol(++$row, 1, g_l('prefs', '[security][customer][slowDownLogin]'))
			->setCol($row, 3, we_html_tools::textInput('newconf[SECURITY_DELAY_FAILED_LOGIN]', self::get_valueS('SECURITY_DELAY_FAILED_LOGIN'), 3, [], 'number', '50px'))
			->setCol($row, 4, 's')
			->setCol(++$row, 1, g_l('prefs', '[security][customer][deleteSession]'))
			->setCol($row, 3, self::getYesNoSelect('SECURITY_DELETE_SESSION'));

		// Build dialog if user has permission
		$cryptkey = self::get_valueS('SECURITY_ENCRYPTION_KEY');
		$encryptionKey = we_html_tools::textInput('newconf[SECURITY_ENCRYPTION_KEY]', ($cryptkey ?: bin2hex(we_customer_customer::cryptGetIV(56))), 112, [], 'text', '20em') . ' (hex)'; //+Button vorhandene Passwörter convertieren

		$storeSessionPassword = (new we_html_select(['name' => 'newconf[SECURITY_SESSION_PASSWORD]', 'class' => 'weSelect']))
			->addOptions([
				(string) we_customer_customer::REMOVE_PASSWORD => g_l('prefs', '[security][storeSessionPassword][type][0]'),
				(string) we_customer_customer::STORE_PASSWORD => g_l('prefs', '[security][storeSessionPassword][type][1]'),
				(string) we_customer_customer::STORE_DBPASSWORD => g_l('prefs', '[security][storeSessionPassword][type][2]'),
				(string) (we_customer_customer::STORE_PASSWORD + we_customer_customer::STORE_DBPASSWORD) => g_l('prefs', '[security][storeSessionPassword][type][3]')
			])
			->selectOption(self::get_valueS('SECURITY_SESSION_PASSWORD'));

		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('perms_customer', '[perm_group_title]'), $customer_table->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[security][encryption][symmetricKey]'), $encryptionKey))
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[security][storeSessionPassword][title]'), $storeSessionPassword->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[security][userPassRegex][title]'), we_html_tools::textInput('newconf[SECURITY_USER_PASS_REGEX]', self::get_valueS('SECURITY_USER_PASS_REGEX'), 100, [
						], 'text', '20em')))
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[security][userPassRegex][desc]'), we_html_tools::textInput('newconf[SECURITY_USER_PASS_DESC]', self::get_valueS('SECURITY_USER_PASS_DESC'), 100, [
					], 'text', '20em')))
			->space(we_html_multiIconBox::SPACE_MED2));
		return self::create_dialog('settings_security', $settings);
	}

	private static function getEmailTab(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}

		$emailSelect = we_html_tools::select('newconf[WE_MAILER]', [
				we_mail_mail::TYPE_PHP => g_l('prefs', '[mailer_php]'),
				we_mail_mail::TYPE_SMTP => g_l('prefs', '[mailer_smtp]')], 1, [self::get_valueS('WE_MAILER')], false, [
				'onchange' => "var el = document.getElementById('smtp_table').style;var el2=document.getElementById('auth_table').style;var elAuth=document.getElementsByName('newconf[SMTP_AUTH]')[0]; if(this.value=='" . we_mail_mail::TYPE_SMTP . "'){ el.display='block'; el2.display=(elAuth.value?'block':'none');}else{ el.display='none';el2.display='none';}"], true, 300);
		// TREE
		$encryptSelect = we_html_tools::select('newconf[SMTP_ENCRYPTION]', ['' => g_l('prefs', '[smtp_encryption_none]'), 'ssl' => g_l('prefs', '[smtp_encryption_ssl]'),
				'tls' => g_l('prefs', '[smtp_encryption_tls]')], 1, [self::get_valueS('SMTP_ENCRYPTION')], false, [], true, 180);
		$row = 0;
		$smtp_table = (new we_html_table(['class' => 'default defaultfont', 'id' => 'smtp_table', 'width' => '300', 'style' => 'display: ' . ((self::get_valueS('WE_MAILER') === we_mail_mail::TYPE_PHP) ? 'none' : 'block') . ';'], 4, 2))
			->setCol($row, 0, g_l('prefs', '[smtp_server]'), ['style' => 'padding-right:10px;'])
			->setCol($row, 1, we_html_tools::textInput('newconf[SMTP_SERVER]', self::get_valueS('SMTP_SERVER'), 180, ['placeholder' => 'mail.example.com'], 'text', '180px'), [
				'style' => 'text-align:right'])
			->setCol(++$row, 0, g_l('prefs', '[smtp_port]'))
			->setCol($row, 1, we_html_tools::textInput('newconf[SMTP_PORT]', self::get_valueS('SMTP_PORT'), 180, [], 'number', '180px'), ['style' => 'text-align:right'])
			->setCol(++$row, 0, g_l('prefs', '[smtp_encryption]'))
			->setCol($row, 1, $encryptSelect, ['style' => 'text-align:right'])
			->setCol(++$row, 0, g_l('prefs', '[smtp_auth]'))
			->setCol($row, 1, we_html_tools::select('newconf[SMTP_AUTH]', [
				'' => g_l('weClass', '[none]'),
				we_mail_TransportSmtp::SMTP_AUTH_PLAIN => 'Plain',
				we_mail_TransportSmtp::SMTP_AUTH_LOGIN => 'Login',
				we_mail_TransportSmtp::SMTP_AUTH_CRAMMD5 => 'CramMd5'
				], 1, self::get_value('SMTP_AUTH'), false, [
				'onchange' => "var el2 = document.getElementById('auth_table').style;el2.display=(this.value?'block':'none');"], true, 180)
		);


		$auth_table = (new we_html_table(['class' => 'default defaultfont', 'id' => 'auth_table', 'style' => 'display: ' . (self::get_valueB('SMTP_AUTH') ? 'block' : 'none') . ';'], 2, 2))
			->setCol(0, 0, g_l('prefs', '[smtp_username]'), ['style' => 'padding-right:10px;'])
			->setCol(0, 1, we_html_tools::textInput('newconf[SMTP_USERNAME]', self::get_valueS('SMTP_USERNAME'), 105, ['placeholder' => g_l('prefs', '[smtp_username]')], 'text', '180px'), [
				'style' => 'text-align:right'])
			->setCol(1, 0, g_l('prefs', '[smtp_password]'), ['style' => 'padding-right:10px;'])
			->setCol(1, 1, we_html_tools::textInput('newconf[SMTP_PASSWORD]', self::get_valueS('SMTP_PASSWORD') ? we_customer_customer::NOPWD_CHANGE : '', 105, [
				'placeholder' => g_l('prefs', '[smtp_password]')], 'password', '180px'), ['style' => 'text-align:right']);

		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('prefs', '[mailer_information]', false, true), we_html_tools::TYPE_INFO, 450, false))))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[mailer_type]'), $emailSelect))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline())
			->add((new we_html_multiIconBoxContentLine('', $smtp_table->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
				->noline())
			->add((new we_html_multiIconBoxContentLine('', $auth_table->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED)
			)
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[smtp_mailFrom]'), we_html_tools::textInput('newconf[SMTP_MAIL_SENDER]', self::get_valueS('SMTP_MAIL_SENDER'), 180, [
					'placeholder' => 'admin@mail.example.com'], 'email', '180px'))));

		return self::create_dialog('settings_email', $settings);
	}

	private static function getVersionTab(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}

		$versionsPrefs = [
			'ctypes' => [
				we_base_ContentTypes::WEDOCUMENT => 'VERSIONING_TEXT_WEBEDITION',
				we_base_ContentTypes::TEMPLATE => 'VERSIONING_TEXT_WETMPL',
				we_base_ContentTypes::OBJECT_FILE => 'VERSIONING_OBJECT',
				we_base_ContentTypes::HTML => 'VERSIONING_TEXT_HTML',
				we_base_ContentTypes::JS => 'VERSIONING_TEXT_JS',
				we_base_ContentTypes::CSS => 'VERSIONING_TEXT_CSS',
				we_base_ContentTypes::TEXT => 'VERSIONING_TEXT_PLAIN',
				we_base_ContentTypes::IMAGE => 'VERSIONING_IMAGE',
				we_base_ContentTypes::VIDEO => 'VERSIONING_VIDEO',
				we_base_ContentTypes::AUDIO => 'VERSIONING_AUDIO',
				we_base_ContentTypes::PDF => 'VERSIONING_PDF',
				we_base_ContentTypes::APPLICATION => 'VERSIONING_SONSTIGE',
				we_base_ContentTypes::XML => 'VERSIONING_TEXT_XML',
				we_base_ContentTypes::HTACCESS => 'VERSIONING_TEXT_HTACCESS',
			],
			'other' => [
				'VERSIONS_TIME_DAYS' => 'VERSIONS_TIME_DAYS',
				'VERSIONS_TIME_WEEKS' => 'VERSIONS_TIME_WEEKS',
				'VERSIONS_TIME_YEARS' => 'VERSIONS_TIME_YEARS',
				'VERSIONS_ANZAHL' => 'VERSIONS_ANZAHL',
				'VERSIONS_CREATE' => 'VERSIONS_CREATE',
				'VERSIONS_CREATE_TMPL' => 'VERSIONS_CREATE_TMPL',
				'VERSIONS_TIME_DAYS_TMPL' => 'VERSIONS_TIME_DAYS_TMPL',
				'VERSIONS_TIME_WEEKS_TMPL' => 'VERSIONS_TIME_WEEKS_TMPL',
				'VERSIONS_TIME_YEARS_TMPL' => 'VERSIONS_TIME_YEARS_TMPL',
				'VERSIONS_ANZAHL_TMPL' => 'VERSIONS_ANZAHL_TMPL'
			]
		];
		// Set selected extension
		$_SESSION['weS']['versions']['logPrefs'] = [];
		foreach($versionsPrefs as $v){
			foreach($v as $val){
				$_SESSION['weS']['versions']['logPrefs'][$val] = self::get_value($val);
			}
		}

		$checkboxes = we_html_forms::checkbox('1', false, 'version_all', g_l('prefs', '[version_all]'), false, '', 'checkAll(this);') . '<br/>';

		foreach($versionsPrefs['ctypes'] as $k => $v){
			$checkboxes .= we_html_forms::checkbox('1', self::get_valueB($v), 'newconf[' . $v . ']', g_l('contentTypes', '[' . $k . ']'), false, '', 'checkAllRevert(this);') . '<br/>';
		}

		$versions_time_days = new we_html_select(['name' => 'newconf[VERSIONS_TIME_DAYS]', 'class' => 'weSelect']);
		$versions_time_days->addOptions([
			'-1' => '',
			(string) we_base_util::SECONDS_DAY => g_l('prefs', '[1_day]')
		]);
		for($x = 2; $x <= 31; $x++){
			$versions_time_days->addOption((string) ($x * we_base_util::SECONDS_DAY), sprintf(g_l('prefs', '[more_days]'), $x));
		}
		$versions_time_days->selectOption(self::get_valueS('VERSIONS_TIME_DAYS'));
		//WINDOW DIMENSIONS

		$versions_time_weeks = new we_html_select(['name' => 'newconf[VERSIONS_TIME_WEEKS]', 'class' => 'weSelect']);
		$versions_time_weeks->addOptions([
			'-1' => '',
			(string) we_base_util::SECONDS_WEEK => g_l('prefs', '[1_week]')
		]);
		for($x = 2; $x <= 52; $x++){
			$versions_time_weeks->addOption((string) ($x * we_base_util::SECONDS_WEEK), sprintf(g_l('prefs', '[more_weeks]'), $x));
		}
		$versions_time_weeks->selectOption(self::get_valueS('VERSIONS_TIME_WEEKS'));

		// Build maximize window
		$versions_time_years = new we_html_select(['name' => 'newconf[VERSIONS_TIME_YEARS]', 'class' => 'weSelect']);
		$versions_time_years->addOptions([
			'-1' => '',
			(string) we_base_util::SECONDS_YEAR => g_l('prefs', '[1_year]')
		]);
		for($x = 2; $x <= 10; $x++){
			$versions_time_years->addOption((string) ($x * we_base_util::SECONDS_YEAR), sprintf(g_l('prefs', '[more_years]'), $x));
		}
		$versions_time_years->selectOption(self::get_valueS('VERSIONS_TIME_YEARS'));
		$versions_anzahl = we_html_tools::textInput('newconf[VERSIONS_ANZAHL]', self::get_valueS('VERSIONS_ANZAHL'), 5, [], 'number', '50px');

		$versions_create_publishing = we_html_forms::radiobutton('1', self::get_valueB('VERSIONS_CREATE'), 'newconf[VERSIONS_CREATE]', g_l('prefs', '[versions_create_publishing]'), true, '', '', false, '');
		$versions_create_always = we_html_forms::radiobutton('0', !self::get_valueB('VERSIONS_CREATE'), 'newconf[VERSIONS_CREATE]', g_l('prefs', '[versions_create_always]'), true, '', '', false, '');
		// Build specify window dimension
		$versions_time_days_tmpl = new we_html_select(['name' => 'newconf[VERSIONS_TIME_DAYS_TMPL]', 'class' => 'weSelect']);
		$versions_time_days_tmpl->addOptions([
			'-1' => '',
			(string) we_base_util::SECONDS_DAY => g_l('prefs', '[1_day]')
		]);
		for($x = 2; $x <= 31; $x++){
			$versions_time_days_tmpl->addOption((string) ($x * we_base_util::SECONDS_DAY), sprintf(g_l('prefs', '[more_days]'), $x));
		}
		$versions_time_days_tmpl->selectOption(self::get_valueS('VERSIONS_TIME_DAYS_TMPL'));

		// Create specify window dimension input
		$versions_time_weeks_tmpl = new we_html_select(['name' => 'newconf[VERSIONS_TIME_WEEKS_TMPL]', 'class' => 'weSelect']);
		$versions_time_weeks_tmpl->addOptions([
			'-1' => '',
			(string) we_base_util::SECONDS_WEEK => g_l('prefs', '[1_week]')
		]);
		for($x = 2; $x <= 52; $x++){
			$versions_time_weeks_tmpl->addOption((string) ($x * we_base_util::SECONDS_WEEK), sprintf(g_l('prefs', '[more_weeks]'), $x));
		}
		$versions_time_weeks_tmpl->selectOption(self::get_valueS('VERSIONS_TIME_WEEKS_TMPL'));

		$versions_time_years_tmpl = new we_html_select(['name' => 'newconf[VERSIONS_TIME_YEARS_TMPL]', 'class' => 'weSelect']);
		$versions_time_years_tmpl->addOptions([
			'-1' => '',
			(string) we_base_util::SECONDS_YEAR => g_l('prefs', '[1_year]')
		]);
		for($x = 2; $x <= 10; $x++){
			$versions_time_years_tmpl->addOption((string) ($x * we_base_util::SECONDS_YEAR), sprintf(g_l('prefs', '[more_years]'), $x));
		}
		$versions_time_years_tmpl->selectOption(self::get_valueS('VERSIONS_TIME_YEARS_TMPL'));
		$versions_anzahl_tmpl = we_html_tools::textInput('newconf[VERSIONS_ANZAHL_TMPL]', self::get_valueS('VERSIONS_ANZAHL_TMPL'), 5, [], 'number', '50px');
		$versions_create_tmpl_publishing = we_html_forms::radiobutton('1', (self::get_valueI('VERSIONS_CREATE_TMPL') == 1), 'newconf[VERSIONS_CREATE_TMPL]', g_l('prefs', '[versions_create_tmpl_publishing]'), true, '', '', false, '');
		$versions_create_tmpl_always = we_html_forms::radiobutton('0', (self::get_valueI('VERSIONS_CREATE_TMPL') == 0), 'newconf[VERSIONS_CREATE_TMPL]', g_l('prefs', '[versions_create_tmpl_always]'), true, '', '', false, '');
		// Build apply current window dimension
		return self::create_dialog('', (new we_html_multiIconBoxContent())
					->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[ContentType]') . we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_activate_text]'), we_html_tools::TYPE_HELP), $checkboxes))
						->space(we_html_multiIconBox::SPACE_BIG)
					)
					->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[versioning_time]') . we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_time_text]'), we_html_tools::TYPE_HELP), $versions_time_days->getHtml() . ' ' . $versions_time_weeks->getHtml() . ' ' . $versions_time_years->getHtml()))
						->space(we_html_multiIconBox::SPACE_BIG)
					)
					->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[versioning_anzahl]'), $versions_anzahl))
						->space(we_html_multiIconBox::SPACE_BIG)
					)
					->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[versioning_create]') . we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_create_text]'), we_html_tools::TYPE_HELP, 0, false),
																																																													 $versions_create_publishing . '<br/>' . $versions_create_always))
						->space(we_html_multiIconBox::SPACE_BIG)
					)
					->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_templates_text]'), we_html_tools::TYPE_INFO, 470, false)))
						->noline()
					)
					->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[versioning_time]'), $versions_time_days_tmpl->getHtml() . ' ' . $versions_time_weeks_tmpl->getHtml() . ' ' . $versions_time_years_tmpl->getHtml()))
						->space(we_html_multiIconBox::SPACE_BIG)
						->noline())
					->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[versioning_anzahl]'), $versions_anzahl_tmpl))
						->noline()
						->space(we_html_multiIconBox::SPACE_BIG)
					)
					->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[versioning_create]'), $versions_create_tmpl_publishing . '<br/>' . $versions_create_tmpl_always))
						->space(we_html_multiIconBox::SPACE_BIG)
					)
				/* ['headline' => g_l('prefs', '[versioning_wizard]') . we_html_tools::alertAttentionBox(g_l('prefs', '[versioning_wizard_text]'), we_html_tools::TYPE_HELP),
				  'html' => '<div style="float:left;">' . we_html_button::create_button('openVersionWizard', 'javascript:openVersionWizard()') . '</div>',

				  ->space(we_html_multiIconBox::SPACE_BIG)
				  ], */
				, -1, '', '');
	}

	// Build final HTML code

	private static function getModulesTab(): string{
		if(!we_base_preferences::userIsAllowed('active_integrated_modules')){
			return '';
		}
		$modInfos = we_modules_info::getAllModules();
		// Build dialog
		$html = '';
		$tmp = [];
		foreach($modInfos as $modInfo){
			if(!$modInfo['alwaysActive']){
				$tmp[$modInfo['text']] = $modInfo;
			}
		}
		ksort($tmp);
		foreach($tmp as $modInfo){
			$onclick = (empty($modInfo['childmodule']) ? '' :
				"if(!this.checked){document.getElementById('newconf[active_integrated_modules][" . $modInfo['childmodule'] . "]').checked=false;}") .
				(empty($modInfo['dependson']) ? '' :
				"if(this.checked){document.getElementById('newconf[active_integrated_modules][" . $modInfo['dependson'] . "]').checked=true;}"
				);
			// Build dialog
			$html .= we_html_forms::checkbox($modInfo['name'], we_modules_info::isActive($modInfo['name']), 'newconf[active_integrated_modules][' . $modInfo['name'] . ']', $modInfo['text'], false, '', $onclick) . '<br/>';
		}
		$html .= we_html_tools::alertAttentionBox(we_html_forms::checkbox('1', false, 'removeNonEmptyTables', g_l('prefs', '[module_activation][removeNonEmptyTables]')), we_html_tools::TYPE_ALERT, '60em', false);

		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[module_activation][headline]') . we_html_tools::alertAttentionBox(g_l('prefs', '[module_activation][information]', false, true), we_html_tools::TYPE_HELP, 0, false),
																																																																	 $html))
			->space(we_html_multiIconBox::SPACE_BIG));

		return self::create_dialog('', $settings, -1);
	}

	private static function getReceipientsTab(): string{
		if(!we_base_preferences::userIsAllowed('FORMMAIL_VIAWEDOC')){
			return '';
		}
		$DB_WE = we_database_base::$inst;

		$settings = new we_html_multiIconBoxContent();
		//FORMMAIL RECIPIENTS
		if(we_base_preferences::userIsAllowed('FORMMAIL_BLOCK')){
			// Build dialog if user has permission
			$settings->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('prefs', '[formmail_information]', false, true), we_html_tools::TYPE_INFO, 450, false))));

			/**
			 * Recipients list
			 */
			$select_box = new we_html_select(['class' => "weSelect", 'name' => "we_recipient", 'size' => 10, 'style' => "width: 340px;height:100px", 'ondblclick' => "edit_recipient();"]);
			// Build select box
			$enabled_buttons = false;

			foreach($DB_WE->getAllFirstq('SELECT ID,Email FROM ' . RECIPIENTS_TABLE . ' ORDER BY Email', false) as $id => $mail){
				$enabled_buttons = true;
				$select_box->addOption((string) $id, (string) $mail);
			}

			$hidden_fields = we_html_element::hiddens(['newconf[formmail_values]' => '',
					'newconf[formmail_deleted]' => '']);

			$editlist_table = (new we_html_table(['class' => 'default'], 1, 2))
				->setCol(0, 0, $hidden_fields . $select_box->getHtml(), ['style' => 'padding-right:10px;'])
				->setCol(0, 1, we_html_button::create_button(we_html_button::ADD, "javascript:add_recipient();") . we_html_button::create_button(we_html_button::EDIT, "javascript:edit_recipient();", '', '', !$enabled_buttons, false) . we_html_button::create_button(we_html_button::DELETE, "javascript:delete_recipient();", '', '', !$enabled_buttons, false), [
				'style' => 'vertical-align:top;']);


			$settings->add(new we_html_multiIconBoxContentLine('', $editlist_table->getHtml()));
		}



		// Build dialog element if user has permission
		if(we_base_preferences::userIsAllowed('FORMMAIL_CONFIRM')){
			$formmail_confirm = (new we_html_select(['name' => 'newconf[FORMMAIL_CONFIRM]', 'style' => "width:88px;", 'class' => "weSelect"]))
				->addOptions([
					'1' => g_l('prefs', '[on]'),
					'0' => g_l('prefs', '[off]')
				])
				->selectOption(self::get_valueB('FORMMAIL_CONFIRM') ? '1' : '0');

			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[formmailConfirm]'), $formmail_confirm->getHtml()))
					->space(we_html_multiIconBox::SPACE_BIG));

			$html = '<table class="default">
							<tr>
								<td>' . self::getYesNoSelect('FORMMAIL_LOG', ['onchange' => "formmailLogOnOff()", 'style' => "width:88px;",]) . '</td>
								<td style="padding-left:10px;">' . we_html_button::create_button(we_html_button::LOG_SHOW, "javascript:we_cmd('we_gui_showFormmailLog')") . '</td>
							</tr>
						</table>';
			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[logFormmailRequests]'), $html))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline());

			$isDisabled = !(self::get_valueB('FORMMAIL_LOG'));

			//Locales
			$formmail_emptylog = (new we_html_select(['name' => "newconf[FORMMAIL_EMPTYLOG]", 'style' => "width:88px;", 'class' => "weSelect"]))
				->addOptions([
					'-1' => g_l('prefs', '[never]'),
					(string) (we_base_util::SECONDS_DAY) => g_l('prefs', '[1_day]'),
					(string) (we_base_util::SECONDS_DAY * 2) => sprintf(g_l('prefs', '[more_days]'), 2),
					(string) (we_base_util::SECONDS_DAY * 4) => sprintf(g_l('prefs', '[more_days]'), 4),
					(string) (we_base_util::SECONDS_WEEK) => g_l('prefs', '[1_week]'),
					(string) (we_base_util::SECONDS_WEEK * 2) => sprintf(g_l('prefs', '[more_weeks]'), 2),
					(string) (we_base_util::SECONDS_WEEK * 4) => sprintf(g_l('prefs', '[more_weeks]'), 4),
					(string) (we_base_util::SECONDS_WEEK * 8) => sprintf(g_l('prefs', '[more_weeks]'), 8),
					(string) (we_base_util::SECONDS_WEEK * 16) => sprintf(g_l('prefs', '[more_weeks]'), 16),
					(string) (we_base_util::SECONDS_WEEK * 32) => sprintf(g_l('prefs', '[more_weeks]'), 32)
				])
				->selectOption(self::get_valueS('FORMMAIL_EMPTYLOG'));

			if($isDisabled){
				$formmail_emptylog->setAttribute("disabled", "disabled");
			}

			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[deleteEntriesOlder]'), $formmail_emptylog->getHtml()))
					->space(we_html_multiIconBox::SPACE_BIG));

			// Create edit list
			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[formmailViaWeDoc]'), self::getYesNoSelect('FORMMAIL_VIAWEDOC', ['style' => "width:88px;",])))
					->space(we_html_multiIconBox::SPACE_BIG));

			// Buttons
			$options = ['onchange' => "formmailBlockOnOff()", 'style' => "width:88px;"];
			if($isDisabled){
				$options['disabled'] = 'disabled';
			}

			$html = '<table class="default">
							<tr>
								<td>' . self::getYesNoSelect('FORMMAIL_BLOCK', $options) . '</td>
								<td style="padding-left:10px;">' . we_html_button::create_button(we_html_button::LOG_SHOW, "javascript:we_cmd('we_gui_showFormmailBlockLog')") . '</td>
							</tr>
						</table>';

			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[blockFormmail]'), $html))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline());
			// Add Locales
			$isDisabled = $isDisabled || (!self::get_valueB("FORMMAIL_BLOCK"));
			// Languages

			$formmail_trials = '<table class="default"><tr><td>' .
				we_html_tools::textInput("newconf[FORMMAIL_TRIALS]", self::get_valueS("FORMMAIL_TRIALS"), 0, [], "text", '88px', false, $isDisabled) .
				'</td></tr></table>';

			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[formmailTrials]'), $formmail_trials))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline());
			// Countries
			if(!$isDisabled){
				$isDisabled = (!self::get_valueB("FORMMAIL_BLOCK"));
			}

			$formmail_span = (new we_html_select(['name' => "newconf[FORMMAIL_SPAN]", 'style' => "width:88px;", 'class' => "weSelect"]))
				->addOptions([
					'60' => g_l('prefs', '[1_minute]'),
					'120' => sprintf(g_l('prefs', '[more_minutes]'), 2),
					'180' => sprintf(g_l('prefs', '[more_minutes]'), 3),
					'300' => sprintf(g_l('prefs', '[more_minutes]'), 5),
					'600' => sprintf(g_l('prefs', '[more_minutes]'), 10),
					'1200' => sprintf(g_l('prefs', '[more_minutes]'), 20),
					'1800' => sprintf(g_l('prefs', '[more_minutes]'), 30),
					'2700' => sprintf(g_l('prefs', '[more_minutes]'), 45),
					'3600' => g_l('prefs', '[1_hour]'),
					'7200' => sprintf(g_l('prefs', '[more_hours]'), 2),
					'14400' => sprintf(g_l('prefs', '[more_hours]'), 4),
					'28800' => sprintf(g_l('prefs', '[more_hours]'), 8),
					(string) (we_base_util::SECONDS_DAY) => sprintf(g_l('prefs', '[more_hours]'), 24)
				])
				->selectOption(self::get_valueS('FORMMAIL_SPAN'));
			if($isDisabled){
				$formmail_span->setAttribute("disabled", "disabled");
			}

			// Button
			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[formmailSpan]'), $formmail_span->getHtml()))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline());
			$formmail_blocktime = (new we_html_select(['name' => "newconf[FORMMAIL_BLOCKTIME]", 'style' => "width:88px;", 'class' => "weSelect"]))
				->addOptions([
					'60' => g_l('prefs', '[1_minute]'),
					'120' => sprintf(g_l('prefs', '[more_minutes]'), 2),
					'180' => sprintf(g_l('prefs', '[more_minutes]'), 3),
					'300' => sprintf(g_l('prefs', '[more_minutes]'), 5),
					'600' => sprintf(g_l('prefs', '[more_minutes]'), 10),
					'1200' => sprintf(g_l('prefs', '[more_minutes]'), 20),
					'1800' => sprintf(g_l('prefs', '[more_minutes]'), 30),
					'2700' => sprintf(g_l('prefs', '[more_minutes]'), 45),
					'3600' => g_l('prefs', '[1_hour]'),
					'7200' => sprintf(g_l('prefs', '[more_hours]'), 2),
					'14400' => sprintf(g_l('prefs', '[more_hours]'), 4),
					'28800' => sprintf(g_l('prefs', '[more_hours]'), 8),
					(string) (we_base_util::SECONDS_DAY) => sprintf(g_l('prefs', '[more_hours]'), 24),
					'-1' => g_l('prefs', '[ever]')
				])
				->selectOption(self::get_valueS("FORMMAIL_BLOCKTIME"));
			if($isDisabled){
				$formmail_blocktime->setAttribute("disabled", "disabled");
			}

			// Build final HTML code
			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[blockFor]'), $formmail_blocktime->getHtml()))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline());
		}

		return self::create_dialog('', $settings, -1);
	}

	private static function getEditorTab(): string{
		//EDITOR PLUGIN

		$attr = ' style="width:150px;"';

		$template_editor_mode = (new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorMode]', 'onchange' => 'displayEditorOptions(this.options[this.options.selectedIndex].value);']))
			->addOptions([
				'textarea' => g_l('prefs', '[editor_plaintext]'),
				'codemirror2' => g_l('prefs', '[editor_javascript2]')
			])
			//$template_editor_mode->addOption('java', g_l('prefs', '[editor_java]'));
			->selectOption(self::get_valueS('editorMode'));

		/**
		 * Editor font settings
		 */
		$template_fonts = [
			'Arial',
			'Andale Mono',
			'Consolas',
			'Courier New',
			'DejaVu Sans Mono',
			'Droid Sans Mono',
			'FreeMono',
			'Helvetica',
			'Inconsolata',
			'Letter Gothic',
			'Liberation Mono',
			'Menlo',
			'Monaco',
			'Mono',
			'Nimbus Mono L',
			'Tahoma',
			'Verdana',
			'serif',
			'sans-serif'];
		$template_font_sizes = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 22, 24, 32, 48, 72];

		$template_editor_font_specify = (self::get_valueS('editorFontname') != '' && self::get_valueS('editorFontname') != 'none');
		$template_editor_font_size_specify = (self::get_valueS('editorFontsize') != '' && self::get_valueS('editorFontsize') != -1);

		// Build specify font
		$template_editor_font_specify_code = we_html_forms::checkbox('1', $template_editor_font_specify, 'newconf[editorFont]', g_l('prefs', '[specify]'), true, 'defaultfont', "if (document.getElementsByName('newconf[editorFont]')[0].checked) { document.getElementsByName('newconf[editorFontname]')[0].disabled = false;document.getElementsByName('newconf[editorFontsize]')[0].disabled = false; } else { document.getElementsByName('newconf[editorFontname]')[0].disabled = true;document.getElementsByName('newconf[editorFontsize]')[0].disabled = true; }");

		$template_editor_font_select_box = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorFontname]', 'style' => 'width: 135px;', ($template_editor_font_specify ? 'enabled' : 'disabled') => ($template_editor_font_specify ? 'enabled' : 'disabled')]);

		foreach($template_fonts as $font){
			$template_editor_font_select_box->addOption($font, $font);
		}
		$template_editor_font_select_box->selectOption($template_editor_font_specify ? self::get_valueS('editorFontname') : 'Courier New');

		$template_editor_font_sizes_select_box = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorFontsize]', 'style' => 'width: 135px;', ($template_editor_font_size_specify ? 'enabled' : 'disabled') => ($template_editor_font_size_specify ? 'enabled' : 'disabled')]);
		foreach($template_font_sizes as $sz){
			$template_editor_font_sizes_select_box->addOption((string) $sz, (string) $sz);
		}
		$template_editor_font_sizes_select_box->selectOption($template_editor_font_specify ? $template_font_sizes[$template_editor_font_specify] : '11');

		$template_editor_font_sizes_select_box->selectOption(self::get_valueS('editorFontsize'));


		$template_editor_font_specify_table = '<table style="margin:0 0 20px 50px;" class="default defaultfont">
	<tr><td' . $attr . '>' . g_l('prefs', '[editor_fontname]') . '</td><td>' . $template_editor_font_select_box->getHtml() . '</td></tr>
	<tr><td' . $attr . '>' . g_l('prefs', '[editor_fontsize]') . '</td><td>' . $template_editor_font_sizes_select_box->getHtml() . '</td></tr>
</table>';
		$template_editor_theme = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorTheme]']);
		foreach(glob(WE_LIB_PATH . 'CodeMirror/theme/*.css', GLOB_NOSORT) as $filename){
			$theme = str_replace(['.css', WE_LIB_PATH . 'CodeMirror/theme/'], '', $filename);
			$template_editor_theme->addOption($theme, $theme);
		}
		$template_editor_theme->selectOption(self::get_valueS('editorTheme'));

		//Build activation of line numbers
		$template_editor_linenumbers_code = we_html_forms::checkbox('1', self::get_valueB('editorLinenumbers'), 'newconf[editorLinenumbers]', g_l('prefs', '[editor_enable]'), true, 'defaultfont', '');
		$template_editor_highlightLine_code = we_html_forms::checkbox('1', self::get_valueB('editorHighlightCurrentLine'), 'newconf[editorHighlightCurrentLine]', g_l('prefs', '[editor_enable]'), true, 'defaultfont', '');

		//Build activation of code completion
		$template_editor_codecompletion_code = we_html_element::hiddens([
				'newconf[editorCodecompletion][WE]' => self::get_valueS('editorCodecompletion-WE'),
				'newconf[editorCodecompletion][htmlTag]' => self::get_valueS('editorCodecompletion-htmlTag'),
				'newconf[editorCodecompletion][htmlDefAttr]' => self::get_valueS('editorCodecompletion-htmlDefAttr'),
				'newconf[editorCodecompletion][htmlAttr]' => self::get_valueS('editorCodecompletion-htmlAttr'),
				'newconf[editorCodecompletion][htmlJSAttr]' => self::get_valueS('editorCodecompletion-htmlJSAttr'),
				/* 'newconf[editorCodecompletion][html5Tag]' => self::get_valueS('editorCodecompletion-html5Tag'),
				  'newconf[editorCodecompletion][html5Attr]' => self::get_valueS('editorCodecompletion-html5Attr') */
			]) .
			we_html_forms::checkbox('1', self::get_valueB('editorCodecompletion-WE'), 'editorCodecompletion0', 'WE-Tags', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][WE]\');') .
			'<br/>' .
			we_html_forms::checkbox('1', self::get_valueB('editorCodecompletion-htmlTag'), 'editorCodecompletion1', 'HTML-Tags', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][htmlTag]\');') .
			'<br/>' .
			we_html_forms::checkbox('1', self::get_valueB('editorCodecompletion-htmlDefAttr'), 'editorCodecompletion2', 'HTML-Default-Attribs', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][htmlDefAttr]\');') .
			'<br/>' .
			we_html_forms::checkbox('1', self::get_valueB('editorCodecompletion-htmlAttr'), 'editorCodecompletion3', 'HTML-Attribs', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][htmlAttr]\');') .
			'<br/>' .
			we_html_forms::checkbox('1', self::get_valueB('editorCodecompletion-htmlJSAttr'), 'editorCodecompletion4', 'HTML-JS-Attribs', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][htmlJSAttr]\');') .
			'<br/>'
		/* we_html_forms::checkbox('1', self::get_valueB('editorCodecompletion-html5Tag'), 'editorCodecompletion5', 'HTML5-Tags', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][html5Tag]\');') .
		  '<br/>' .
		  we_html_forms::checkbox('1', self::get_valueB('editorCodecompletion-html5Attr'), 'editorCodecompletion6', 'HTML5-Attribs', true, 'defaultfont', 'set_xhtml_field(this.checked,\'newconf[editorCodecompletion][html5Attr]\');') */;


		$template_editor_tabstop_code = '<table class="default defaultfont">
				<tr><td colspan="2">' .
			we_html_forms::checkbox('1', self::get_valueB('editorShowTab'), 'editorShowTab', g_l('prefs', '[editor_tabstop]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorShowTab]\');') .
			we_html_element::hidden('newconf[editorShowTab]', self::get_valueS('editorShowTab')) .
			'</td></tr>
				<tr><td style="width:200px;">' . g_l('prefs', '[editor_tabSize]') . '</td><td>' . we_html_tools::textInput("newconf[editorTabSize]", self::get_valueS('editorTabSize'), 2, [
				], "number", '135px') . '</td></tr>
				<tr><td colspan="2">' .
			we_html_forms::checkbox('1', self::get_valueB('editorAutoIndent'), 'editorAutoIndent', g_l('prefs', '[editor_autoindent]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorAutoIndent]\');') .
			we_html_element::hidden('newconf[editorAutoIndent]', self::get_valueS('editorAutoIndent')) .
			'</td></tr>
				<tr><td colspan="2">' .
			we_html_forms::checkbox('1', self::get_valueB('editorIndentSpaces'), 'editorIndentSpaces', g_l('prefs', '[editor][indentSpaces]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorIndentSpaces]\');') .
			we_html_element::hidden('newconf[editorIndentSpaces]', self::get_valueS('editorIndentSpaces')) .
			'</td></tr>
				<tr><td colspan="2">' .
			we_html_forms::checkbox('1', self::get_valueB('editorShowSpaces'), 'editorShowSpaces', g_l('prefs', '[editor][showSpaces]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorShowSpaces]\');') .
			we_html_element::hidden('newconf[editorShowSpaces]', self::get_valueS('editorShowSpaces')) .
			'</td></tr>
			</table>';

		$template_editor_Wrap_code = we_html_forms::checkbox('1', self::get_valueB('editorWrap'), 'editorWrap', g_l('prefs', '[editor_enable]'), true, '', 'set_xhtml_field(this.checked,\'newconf[editorWrap]\');') .
			we_html_element::hidden('newconf[editorWrap]', self::get_valueS('editorWrap'));


		$template_editor_tooltips_code = we_html_forms::checkbox('1', self::get_valueB('editorTooltips'), 'newconf[editorTooltips]', g_l('prefs', '[editorTooltips]'), true, '', '') .
			'<br/>' .
			we_html_forms::checkbox('1', self::get_valueB('editorTooltipsIDs'), 'newconf[editorTooltipsIDs]', g_l('prefs', '[editorTooltipsIDs]'), true, '', '');

		$template_editor_tooltip_font_specify = (self::get_valueS('editorTooltipFontname') != '' && self::get_valueS('editorTooltipFontname') != 'none');
		$template_editor_tooltip_font_size_specify = (self::get_valueS('editorTooltipFontsize') != '' && self::get_valueS('editorTooltipFontsize') != '-1');

		// Build specify font
		$template_editor_tooltip_font_specify_code = we_html_forms::checkbox('1', $template_editor_tooltip_font_specify, 'newconf[editorTooltipFont]', g_l('prefs', '[specify]'), true, '', 'if (document.getElementsByName(\'newconf[editorTooltipFont]\')[0].checked) { document.getElementsByName(\'newconf[editorTooltipFontname]\')[0].disabled = false;document.getElementsByName(\'newconf[editorTooltipFontsize]\')[0].disabled = false; } else { document.getElementsByName(\'newconf[editorTooltipFontname]\')[0].disabled = true;document.getElementsByName(\'newconf[editorTooltipFontsize]\')[0].disabled = true; }');

		$template_editor_tooltip_font_select_box = new we_html_select(['class' => 'weSelect', 'name' => 'newconf[editorTooltipFontname]', 'style' => 'width: 135px;',
			($template_editor_tooltip_font_specify ? 'enabled' : 'disabled') => ($template_editor_tooltip_font_specify ? 'enabled' : 'disabled')]);

		foreach($template_fonts as $font){
			$template_editor_tooltip_font_select_box->addOption($font, $font);
		}
		$template_editor_tooltip_font_select_box->selectOption($template_editor_tooltip_font_specify ? self::get_valueS('editorTooltipFontname') : 'Tahoma');

		$template_editor_tooltip_font_sizes_select_box = new we_html_select(['class' => 'weSelect editor editor_codemirror2', 'name' => 'newconf[editorTooltipFontsize]',
			'style' => 'width: 135px;', ($template_editor_tooltip_font_size_specify ? 'enabled' : 'disabled') => ($template_editor_tooltip_font_size_specify ? 'enabled' : 'disabled')]);
		$template_toolfont_sizes = [10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20];
		foreach($template_toolfont_sizes as $sz){
			$template_editor_tooltip_font_sizes_select_box->addOption((string) $sz, (string) $sz);
		}
		$template_editor_tooltip_font_sizes_select_box->selectOption($template_editor_tooltip_font_specify ? self::get_valueS('editor_tooltip_font_size') : '11');
		$template_editor_tooltip_font_specify_table = '<table style="margin:0 0 20px 50px;" class="default defaultfont">
				<tr><td' . $attr . '>' . g_l('prefs', '[editor_fontname]') . '</td><td>' . $template_editor_tooltip_font_select_box->getHtml() . '</td></tr>
				<tr><td' . $attr . '>' . g_l('prefs', '[editor_fontsize]') . '</td><td>' . $template_editor_tooltip_font_sizes_select_box->getHtml() . '</td></tr>
			</table>';

		//Build activation of integration of documentation
		$template_editor_autoClose = we_html_forms::checkbox('1', self::get_valueB('editorDocuintegration'), 'newconf[editorDocuintegration]', g_l('prefs', '[editor_enable]'), true, '', '') .
//remove fonts not available
			we_html_element::jsScript(WE_LIB_DIR . 'fontdetect/fontdetect.js', 'checkFonts()');
		$settings = (new we_html_multiIconBoxContent())
			//['headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[editor_information]'), we_html_tools::TYPE_INFO, 480, false),],
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[editor_mode]'), $template_editor_mode->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[editor_font]'), $template_editor_font_specify_code . $template_editor_font_specify_table))
				->class('editor editor_codemirror2 editor_textarea')
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[editor_theme]'), $template_editor_theme->getHtml()))
				->class('editor editor_codemirror2')
				->space(we_html_multiIconBox::SPACE_MED2))
//				array('class' => 'editor editor_java', 'headline' => g_l('prefs', '[editor_highlight_colors]'), 'html' => $template_editor_font_color_checkbox . $template_editor_font_color_table, 'space' => we_html_multiIconBox::SPACE_MED2),
			//array('class'=>'editor editor_codemirror2','headline' => g_l('prefs', '[editor_docuclick]'), 'html' => $template_editor_docuintegration_code, 'space' => we_html_multiIconBox::SPACE_MED2),
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[editor_linenumbers]'), $template_editor_linenumbers_code))
				->class('editor editor_codemirror2')
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[editor_highlightLine]'), $template_editor_highlightLine_code))
				->class('editor editor_codemirror2')
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('global', '[wrapcheck]'), $template_editor_Wrap_code))
				->class('editor editor_codemirror2 editor_textarea')
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '	[editor][indent]'), $template_editor_tabstop_code))
				->class('editor editor_codemirror2 editor_textarea')
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[editor_completion]'), $template_editor_codecompletion_code))
				->class('editor editor_codemirror2')
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[editor_tooltips]'), $template_editor_tooltips_code . '<br/>' . $template_editor_tooltip_font_specify_code . '<br/>' . $template_editor_tooltip_font_specify_table))->space(we_html_multiIconBox::SPACE_MED2)->class('editor editor_codemirror2'))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[editor_autoCloseTags]'), $template_editor_autoClose))
			->class('editor editor_codemirror2')
			->space(we_html_multiIconBox::SPACE_MED2))
		;

		return self::create_dialog('', $settings);
	}

	//FORMMAIL RECIPIENTS
	// Build dialog if user has permission
	private static function getLanguagesTab(): string{
		if(!we_base_preferences::userIsAllowed('locale_default') && we_base_preferences::userIsAllowed('locale_locales')){
			return '';
		}
		$default = self::get_valueS('locale_default');
		$locales = we_base_country::getWeFrontendLanguagesForBackend();

		self::$jsCmd->addCmd('initLocale', $default);

		$hidden_fields = we_html_element::hidden('newconf[locale_default]', $default, 'locale_default') .
			we_html_element::hidden('newconf[locale_locales]', implode(',', array_keys($locales)), 'locale_locales');


		$select_box = (new we_html_select(['class' => 'weSelect', 'name' => 'locale_temp_locales', 'size' => 10, 'id' => 'locale_temp_locales', 'style' => 'width: 340px']))
			->addOptions($locales);
		// Create needed hidden fields
		$enabled_buttons = (count($locales) > 0);

		// Create edit list

		$editlist_table = (new we_html_table(['class' => 'default'], 1, 2))
			->setCol(0, 0, $hidden_fields . $select_box->getHtml(), ['style' => 'padding-right:10px;'])
			->setCol(0, 1, we_html_button::create_button('default', 'javascript:defaultLocale()', '', '', !$enabled_buttons) . we_html_button::create_button(we_html_button::DELETE, 'javascript:deleteLocale()'), [
			'style' => 'vertical-align:top']);
		// Build dialog if user has permission
		$Languages = we_base_country::g_l_a('languages', '');
		$TopLanguages = [
			'~de' => $Languages['de'],
			'~nl' => $Languages['nl'],
			'~en' => $Languages['en'],
			'~fi' => $Languages['fi'],
			'~fr' => $Languages['fr'],
			'~pl' => $Languages['pl'],
			'~ru' => $Languages['ru'],
			'~es' => $Languages['es'],
		];
		asort($Languages);
		asort($TopLanguages);
		$TopLanguages[''] = '---';

		$languages = (new we_html_select(['name' => 'newconf[locale_language]', 'id' => 'locale_language', 'style' => 'width: 139px', 'class' => 'weSelect']))
			->addOptions(array_merge($TopLanguages, $Languages));
		// formmail stuff

		$Countries = we_base_country::g_l_a('countries', '');
		$TopCountries = [
			'~DE' => $Countries['DE'],
			'~CH' => $Countries['CH'],
			'~AT' => $Countries['AT'],
			'~NL' => $Countries['NL'],
			'~GB' => $Countries['GB'],
			'~US' => $Countries['US'],
			'~FI' => $Countries['FI'],
			'~FR' => $Countries['FR'],
			'~PL' => $Countries['PL'],
			'~RU' => $Countries['RU'],
			'~ES' => $Countries['ES'],
		];
		asort($Countries);
		asort($TopCountries);
		$TopCountries['~'] = '---';

		$countries = (new we_html_select(['name' => 'newconf[locale_country]', 'id' => 'locale_country', 'style' => 'width: 139px', 'class' => 'weSelect']))
			->addOptions(array_merge(['' => ''], $TopCountries, $Countries));

		$add_button = we_html_button::create_button(we_html_button::ADD, 'javascript:addLocale()');


		$add_html = g_l('prefs', '[locale_languages]') . we_html_element::br() .
			$languages->getHtml() . we_html_element::br(2) .
			g_l('prefs', '[locale_countries]') . we_html_element::br() .
			$countries->getHtml() . we_html_element::br(2) .
			$add_button;

		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[tab][language]'), we_html_tools::alertAttentionBox(g_l('prefs', '[locale_information]', false, true), we_html_tools::TYPE_INFO, 450, false, 50) . we_html_element::br() . $editlist_table->getHtml()))
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[locale_add]'), $add_html))
				->space(we_html_multiIconBox::SPACE_MED2))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[langlink_headline]'), ''))
				->noline())
			->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('prefs', '[langlink_information]', false, true), we_html_tools::TYPE_INFO, 450, false, 50)))
				->space(we_html_multiIconBox::SPACE_MED2)
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[langlink_support]'), self::getTrueFalseSelect('LANGLINK_SUPPORT')))
			->space(we_html_multiIconBox::SPACE_MED2)
			->noline());

		return self::create_dialog('', $settings);
	}

	private static function getCountryTab(): string{
		if(!we_base_preferences::userIsAllowed('WE_COUNTRIES_DEFAULT')){
			return '';
		}

		$countries_default = we_html_tools::textInput('newconf[WE_COUNTRIES_DEFAULT]', self::get_valueS('WE_COUNTRIES_DEFAULT'), 0, [], 'text', '225px');

		$lang = explode('_', we_base_WE::$Language);
		$langcode = (string) array_search($lang[0], we_base_country::getWELangs(), true) ?: 'de';
		$countrycode = (string) array_search($langcode, we_base_country::getWECountries(), true) ?: 'DE';
		$supported = we_base_country::getTranslationList(we_base_country::TERRITORY, $langcode);
		$oldLocale = setlocale(LC_ALL, NULL);
		setlocale(LC_ALL, $langcode . '_' . $countrycode . '.UTF-8');
		asort($supported, SORT_LOCALE_STRING);
		if($oldLocale){
			setlocale(LC_ALL, $oldLocale);
		}
		$countries_top = explode(',', self::get_valueS('WE_COUNTRIES_TOP'));
		$countries_shown = explode(',', self::get_valueS('WE_COUNTRIES_SHOWN'));
		$tabC = (new we_html_table(['style' => 'border:1px solid black', 'class' => 'defaultfont'], 1, 4))
			->addRow(new we_html_tr([
				new we_html_td(g_l('prefs', '[countries_country]'), ['class' => 'bold']),
				new we_html_td(g_l('prefs', '[countries_top]'), ['class' => 'bold']),
				new we_html_td(g_l('prefs', '[countries_show]'), ['class' => 'bold']),
				new we_html_td(g_l('prefs', '[countries_noshow]'), ['class' => 'bold']),
		]));
		foreach($supported as $countrycode => $country){
			$tabC->addRow(new we_html_tr([
					new we_html_td((string) $country),
					new we_html_td('<input type="radio" name="newconf[countries][' . $countrycode . ']" value="2" ' . (in_array($countrycode, $countries_top, true) ? 'checked="checked"' : '') . ' > '),
					new we_html_td('<input type="radio" name="newconf[countries][' . $countrycode . ']" value="1" ' . (in_array($countrycode, $countries_shown, true) ? 'checked="checked"' : '') . ' > '),
					new we_html_td('<input type="radio" name="newconf[countries][' . $countrycode . ']" value="0" ' . (!in_array($countrycode, $countries_top, true) && !in_array($countrycode, $countries_shown, true) ? 'checked' : '') . ' > '),
			]));
		}
		// formmail only via we doc //
		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[countries_headline]'), we_html_tools::alertAttentionBox(g_l('prefs', '[countries_information]', false, true), we_html_tools::TYPE_INFO, 450, false)))
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[countries_default]'), $countries_default))
				->space(we_html_multiIconBox::SPACE_BIG)
				->noline())
			->add((new we_html_multiIconBoxContentLine('', $tabC->getHtml()))
			->noline());

		return self::create_dialog('', $settings);
	}

	private static function getCacheTab(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}
		$avail = array_filter([
			we_cache_backendNone::BACKEND_NONE => g_l('prefs', '[cache_type_none]'),
			we_cache_backendNone::BACKEND_MEMCAHCED => extension_loaded('memcached') ? 'Memcached Server' : '',
			we_cache_backendNone::BACKEND_REDIS => extension_loaded('redis') ? 'Redis Server-Array' : ''
		]);
		if(empty($avail)){
			return we_html_tools::alertAttentionBox(g_l('prefs', '[memcached][notFound]'), we_html_tools::TYPE_ALERT, 450, false);
		}

		$type = we_html_tools::select('newconf[MEMCACHE_BACKEND]', $avail, 1, [self::get_valueS('MEMCACHE_BACKEND')]);
		$server = we_html_tools::textInput('newconf[MEMCACHE_SERVERS]', self::get_valueS('MEMCACHE_SERVERS'), 0, ['autocomplete' => 'off'], 'text', '500px', false);
		$db = we_html_tools::textInput('newconf[MEMCACHE_BACKEND_DB]', self::get_valueS('MEMCACHE_BACKEND_DB'), 0, ['autocomplete' => 'off'], 'number', '4em', false);

		$dbCache = self::getYesNoSelect('MEMCACHE_DATABASE');

		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[memcache][type]', false, true), $type))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[memcached][servers]', false, true), $server))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[redis][database]', false, true), $db))->space(we_html_multiIconBox::SPACE_MED2)->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[memcached][DBCache]', false, true), $dbCache))->space(we_html_multiIconBox::SPACE_MED2))
		;

		return self::create_dialog('', $settings, -1);
	}

	private static function getDefaultSettingsTab(): string{
		if(!we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR])){
			return '';
		}
		$weSuggest = we_gui_suggest::getInstance();
		/**
		 * inlineedit setting
		 */
		$commands_default_tmp = we_html_tools::select('tmp_commands', we_wysiwyg_editor::getEditorCommands(false), 1, [], false, ['onchange' => "var elem=document.getElementById('commands_default_id'); var txt = this.options[this.selectedIndex].text; if(elem.value.split(',').indexOf(txt)==-1){elem.value=(elem.value) ? (elem.value + ',' + txt) : txt;}this.selectedIndex=-1"]);
		$COMMANDS_DEFAULT = we_html_tools::textInput('newconf[COMMANDS_DEFAULT]', self::get_valueS('COMMANDS_DEFAULT'), 0, ['id' => 'commands_default_id'], 'text', '225px');

		$CSSAPPLYTO_DEFAULT = (new we_html_select(['name' => 'newconf[CSSAPPLYTO_DEFAULT]', 'class' => 'weSelect']))
			->addOptions([
				'all' => 'all',
				'around' => 'around',
				'wysiwyg' => 'wysiwyg',
			])
			->selectOption(self::get_valueS('CSSAPPLYTO_DEFAULT') ?: 'around');

		$weSuggest->setAcId('DocNoOF')
			->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT])
			->setInput('imagestartid_default_text', (IMAGESTARTID_DEFAULT ? id2path(IMAGESTARTID_DEFAULT) : ''))
			->setMaxResults(20)
			->setResult('newconf[IMAGESTARTID_DEFAULT]', (string) (IMAGESTARTID_DEFAULT ?: 0))
			->setSelector(we_gui_suggest::DirSelector)
			->setWidth(226)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_directory', {id:document.forms[0].elements['newconf[IMAGESTARTID_DEFAULT]'].value,table:WE().consts.tables.FILE_TABLE, targetID:'newconf[IMAGESTARTID_DEFAULT]',targetName:'imagestartid_default_text'})"))
			->setTrashButton(we_html_button::create_button(we_html_button::TRASH, "javascript:document.forms[0].elements['newconf[IMAGESTARTID_DEFAULT]'].value = 0;document.forms[0].elements.imagestartid_default_text.value=''"));
		$settings = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[inlineedit_default]', false, true), self::getTrueFalseSelect('INLINEEDIT_DEFAULT')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[imagestartid_default]', false, true), $weSuggest->getHTML()))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[commands_default]', false, true), '<div>' . $commands_default_tmp . '</div><div style="margin-top:4px">' . $COMMANDS_DEFAULT . '</div>'))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[removefirstparagraph_default]', false, true), self::getTrueFalseSelect('REMOVEFIRSTPARAGRAPH_DEFAULT')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[showinputs_default]', false, true), self::getTrueFalseSelect('SHOWINPUTS_DEFAULT')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[hidenameattribinweimg_default]'), self::getYesNoSelect('HIDENAMEATTRIBINWEIMG_DEFAULT')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[hidenameattribinweform_default]'), self::getYesNoSelect('HIDENAMEATTRIBINWEFORM_DEFAULT')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[replaceacronym]'), self::getYesNoSelect('REPLACEACRONYM')))
				->space(we_html_multiIconBox::SPACE_BIG))
			->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[cssapplyto_default]', false, true), $CSSAPPLYTO_DEFAULT->getHtml()))
			->space(we_html_multiIconBox::SPACE_BIG));

		return self::create_dialog('', $settings, -1);
	}

	private static function getUITab(): string{
		$weSuggest = we_gui_suggest::getInstance();

		$settings = new we_html_multiIconBoxContent();


		$language_directory = dir(WE_INCLUDES_PATH . 'language');
		$language = [];

		while(false !== ($entry = $language_directory->read())){
			if($entry != '.' && $entry != '..'){
				if(is_dir(WE_INCLUDES_PATH . 'language/' . $entry)){
					$language[$entry] = $entry;
				}
			}
		}
		global $languages;

		if(!empty($language)){ // Build language select box
			$languages = new we_html_select(['name' => 'newconf[Language]', 'class' => 'weSelect', 'onchange' => "document.getElementById('langnote').style.display='block'"]);
			foreach($language as $key => $value){
				$languages->addOption($key, $value);
			}
			$languages->selectOption(self::get_valueS('Language'));
			// Check Proxy settings  ...
			$langNote = '<div id="langnote" style="padding: 5px; background-color: rgb(221, 221, 221); width: 190px; display:none">
<table style="width:100%">
<tbody>
<tr>
<td style="padding-right: 10px;vertical-align:top">
	<span class="fa-stack fa-lg" style="font-size: 14px;color:#007de3;"><i class="fa fa-circle fa-stack-2x" ></i><i class="fa fa-info fa-stack-1x fa-inverse"></i></span>
</td>
<td class="middlefont">' . g_l('prefs', '[language_notice]') . '
</td>
</tr>
</tbody>
</table>
</div>';

			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[choose_language]'), $languages->getHtml() . we_html_element::br(2) . $langNote))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline());
		} else { // Just one Language Installed, no select box needed
			$languages = current($language) ?: '';

			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[choose_language]'), $languages))
					->space(we_html_multiIconBox::SPACE_BIG)
					->noline());
		}
		// Build dialog if user has permission

		$cockpit_amount_columns = new we_html_select(['name' => 'newconf[cockpit_amount_columns]', 'class' => 'weSelect']);
		for($i = 1; $i <= 10; $i++){
			$cockpit_amount_columns->addOption((string) $i, (string) $i);
		}
		$cockpit_amount_columns->selectOption(self::get_valueS('cockpit_amount_columns'));
		$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[cockpit_amount_columns]'), $cockpit_amount_columns->getHtml()))
				->space(we_html_multiIconBox::SPACE_BIG));

		/*		 * **************************************************************
		 * SEEM
		 * *************************************************************** */

		if(we_base_preferences::userIsAllowed('WE_SEEM')){

			$seem_disabler = we_html_forms::checkbox('1', self::get_valueI('WE_SEEM') == 0, 'newconf[WE_SEEM]', g_l('prefs', '[seem_deactivate]'));


			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[seem]'), $seem_disabler))
					->space(we_html_multiIconBox::SPACE_BIG));
		}


		if(we_base_preferences::userIsAllowed('seem_start_type')){
			$document_path = $object_path = '';
			$document_id = $object_id = 0;

			switch(self::get_valueS('seem_start_type')){
				default:
					$seem_start_type = 'none';
					break;
				case 'cockpit':
					$_SESSION['prefs']['seem_start_file'] = 0;
					$seem_start_type = 'cockpit';
					break;
				// Build select box

				case 'object':
					$seem_start_type = 'object';
					$object_id = self::get_valueI('seem_start_file');
					$object_path = $object_id ? id2path($object_id, OBJECT_FILES_TABLE) : '';
					break;
				/* case 'weapp':
				  $seem_start_type = 'weapp';
				  if(self::get_valueS('seem_start_weapp') != ''){
				  $seem_start_weapp = self::get_valueS('seem_start_weapp');
				  }
				  //FILE EXTENSIONS
				  break; */
				// Get webEdition extensions
				case 'document':
					$seem_start_type = 'document';
					$document_id = self::get_valueI('seem_start_file');
					$document_path = $document_id ? id2path($document_id, FILE_TABLE) : '';
					break;
			}

			$start_type = new we_html_select(['name' => 'newconf[seem_start_type]', 'class' => 'weSelect', 'id' => 'seem_start_type', 'onchange' => "show_seem_chooser(this.value);"]);

			$showStartType = false;
			$permitedStartTypes = [''];
			$start_type->addOption('0', '-');
			$seem_cockpit_selectordummy = "<div id='selectordummy' style='height:24px;'>&nbsp;</div>";
			if(we_base_permission::hasPerm([we_base_perm::WORK_QUICKSTART_CAN_SEE])){
				$start_type->addOption('cockpit', g_l('prefs', '[seem_start_type_cockpit]'));
				$showStartType = true;
				$permitedStartTypes[] = 'cockpit';
			}

			$seem_document_chooser = '';
			if(we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
				$start_type->addOption('document', g_l('prefs', '[seem_start_type_document]'));
				$showStartType = true;


				$weSuggest->setAcId('Doc')
					->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML, we_base_ContentTypes::IMAGE])
					->setInput('seem_start_document_name', $document_path, [], self::get_valueB('seem_start_file'))
					->setMaxResults(20)
					->setRequired(true)
					->setResult('seem_start_document', (string) $document_id)
					->setSelector(we_gui_suggest::DocSelector)
					->setWidth(150)
					->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:select_seem_start()', '', '', false, true));
				//	array('headline' => '', 'html' => we_html_tools::alertAttentionBox(g_l('prefs', '[extensions_information]'), we_html_tools::TYPE_INFO, 450, false), ),
				$seem_document_chooser = we_html_element::span(['id' => 'span_seem_start_document', 'style' => 'display:none'], $weSuggest->getHTML());
				$permitedStartTypes[] = 'document';
			}
			$seem_object_chooser = '';
			if(we_modules_info::isActive(we_modules_info::OBJECT) && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
				$start_type->addOption('object', g_l('prefs', '[seem_start_type_object]'));
				$showStartType = true;

				// Build db charset select box
				$weSuggest->setAcId('Obj')
					->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])
					->setInput('seem_start_object_name', $object_path, [], self::get_valueB('seem_start_file'))
					->setMaxResults(20)
					->setRequired(true)
					->setResult('seem_start_object', (string) $object_id)
					->setSelector(we_gui_suggest::DocSelector)
					->setTable(OBJECT_FILES_TABLE)
					->setWidth(150)
					->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:select_seem_start()', '', '', false, true));

				$seem_object_chooser = we_html_element::span(['id' => 'span_seem_start_object', 'style' => 'display:none'], $weSuggest->getHTML());
				$permitedStartTypes[] = 'object';
			}
			/* $start_weapp = new we_html_select(['name' => 'newconf[seem_start_weapp]', 'class' => 'weSelect', 'id' => 'seem_start_weapp']);

			  $seem_weapp_chooser = '';
			  if($start_weapp->getOptionNum()){
			  $start_type->addOption('weapp', g_l('prefs', '[seem_start_type_weapp]'));
			  if(!empty($seem_start_weapp)){
			  $start_weapp->selectOption($seem_start_weapp);
			  }
			  $weAPPSelector = $start_weapp->getHtml();
			  $seem_weapp_chooser = we_html_element::span(['id' => 'seem_start_weapp', 'style' => 'display:none'], $weAPPSelector);
			  $permitedStartTypes[] = 'weapp';
			  } */

			// Check authentication settings  ...
			if($showStartType){
				if(in_array($seem_start_type, $permitedStartTypes)){
					$start_type->selectOption($seem_start_type);
				} else { //  gd lib ist nicht installiert
					$seem_start_type = $permitedStartTypes[0];
				}
				$seem_html = (new we_html_table(['class' => 'default defaultfont'], 2, 1))
					->setCol(0, 0, $start_type->getHtml())
					->setCol(1, 0, $seem_cockpit_selectordummy . $seem_document_chooser . $seem_object_chooser, ['style' => 'padding-top:5px;']);
				$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[seem_startdocument]'), $seem_html->getHtml()))->space(we_html_multiIconBox::SPACE_BIG));
				self::$jsCmd->addCmd('show_seem_chooser', $seem_start_type);
			}
		}
		$val = self::get_valueI('message_reporting');
		// Build dialog if user has permission
		$html = implode(we_html_element::br(), ['<input type="hidden" id="message_reporting" name="newconf[message_reporting]" value="' . $val . '" />' .
			we_html_forms::checkbox((string) we_base_util::WE_MESSAGE_ERROR, true, 'message_reporting_errors', g_l('prefs', '[message_reporting][show_errors]'), false, '', "handle_message_reporting_click();", true),
			we_html_forms::checkbox((string) we_base_util::WE_MESSAGE_WARNING, ($val & we_base_util::WE_MESSAGE_WARNING) > 0, 'message_reporting_warnings', g_l('prefs', '[message_reporting][show_warnings]'), false, '', "handle_message_reporting_click();"),
			we_html_forms::checkbox((string) we_base_util::WE_MESSAGE_NOTICE, ($val & we_base_util::WE_MESSAGE_NOTICE) > 0, 'message_reporting_notices', g_l('prefs', '[message_reporting][show_notices]'), false, '', "handle_message_reporting_click();")
		]);

		$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[message_reporting][headline]') . we_html_tools::alertAttentionBox(g_l('prefs', '[message_reporting][information]', false, true), we_html_tools::TYPE_HELP, 0, false),
																																																																					 $html))
				->space(we_html_multiIconBox::SPACE_BIG));

		/*		 * *******************************************************
		 * Sidebar
		 * ******************************************************* */
		if(we_base_preferences::userIsAllowed('SIDEBAR_DISABLED')){

			$sidebar_disable = self::get_valueB('SIDEBAR_DISABLED');
			$sidebar_show = ($sidebar_disable) ? 'none' : 'block';

			$sidebar_id = self::get_valueI('SIDEBAR_DEFAULT_DOCUMENT');
			$sidebar_path = $sidebar_id ? id2path($sidebar_id, FILE_TABLE) : '';

			$sidebar_disabler = we_html_forms::checkbox('1', $sidebar_disable, 'newconf[SIDEBAR_DISABLED]', g_l('prefs', '[sidebar_deactivate]'), false, '', "document.getElementById('sidebar_options').style.display=(this.checked?'none':'block');");

			// Create checkboxes
			$sidebar_show_on_startup = we_html_forms::checkbox('1', self::get_valueB('SIDEBAR_SHOW_ON_STARTUP'), 'newconf[SIDEBAR_SHOW_ON_STARTUP]', g_l('prefs', '[sidebar_show_on_startup]'), false, '', '');

			// Error types
			$sidebar_width = we_html_tools::textInput('newconf[SIDEBAR_DEFAULT_WIDTH]', self::get_valueS('SIDEBAR_DEFAULT_WIDTH'), 255, ['onchange' => "if ( isNaN( this.value ) ||  parseInt(this.value) < 100 ) { this.value=100; };"], 'number', '90px');
			$sidebar_width_chooser = we_html_tools::select('tmp_sidebar_width', [0 => '', 100 => 100, 150 => 150, 200 => 200, 250 => 250, 300 => 300, 350 => 350, 400 => 400], 1, [
					], false, [
					'onchange' => "document.forms[0].elements['newconf[SIDEBAR_DEFAULT_WIDTH]'].value=this.options[this.selectedIndex].value;this.selectedIndex=-1;"], true, 100);
			// Create checkboxes
			$weSuggest->setAcId('SidebarDoc')
				->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::WEDOCUMENT])
				->setInput('ui_sidebar_file_name', $sidebar_path)
				->setMaxResults(20)
				->setResult('newconf[SIDEBAR_DEFAULT_DOCUMENT]', (string) $sidebar_id)
				->setSelector(we_gui_suggest::DocSelector)
				->setWidth(150)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, 'javascript:selectSidebarDoc()'));

			// Create checkboxes
			$sidebar_html2 = (new we_html_table(['class' => 'default defaultfont', 'id' => 'sidebar_options', 'style' => 'display:' . $sidebar_show], 8, 2))
				->setColAttributes(0, 0, ['colspan' => '2', 'height' => '10px'])
				->setCol(1, 0, $sidebar_show_on_startup, ['colspan' => '2', 'height' => '10px'])
				->setColAttributes(2, 0, ['colspan' => '2', 'height' => '10px'])
				->setCol(3, 0, g_l('prefs', '[sidebar_width]'), ['colspan' => '2'])
				->setCol(4, 0, $sidebar_width)
				->setCol(4, 1, $sidebar_width_chooser, ['style' => 'padding-left:10px;'])
				->setColAttributes(5, 0, ['colspan' => '2', 'height' => '10px'])
				->setCol(6, 0, g_l('prefs', '[sidebar_document]'), ['colspan' => '2'])
				->setCol(7, 0, $weSuggest->getHTML(), ['colspan' => '2']);


			$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[sidebar]'), $sidebar_disabler . $sidebar_html2->getHtml()))
					->space(we_html_multiIconBox::SPACE_BIG));
		}


		$tree_count = self::get_valueS('default_tree_count');
		$file_tree_count = (new we_html_select(['name' => 'newconf[default_tree_count]', 'class' => 'weSelect']))
			->addOption('0', g_l('prefs', '[all]'));

		for($i = 10; $i < 51; $i += 10){
			$file_tree_count->addOption((string) $i, (string) $i);
		}

		for($i = 100; $i < 501; $i += 100){
			$file_tree_count->addOption((string) $i, (string) $i);
		}
		$file_tree_count->selectOption($tree_count);

		/* if(!$file_tree_count->selectOption($tree_count)){
		  $file_tree_count->addOption($tree_count, $tree_count);

		  $file_tree_count->selectOption($tree_count);
		  } */

		$settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[tree_title]') . we_html_tools::alertAttentionBox(g_l('prefs', '[tree_count_description]'), we_html_tools::TYPE_HELP),
																																																													$file_tree_count->getHtml()))
				->space(we_html_multiIconBox::SPACE_BIG));


		/*

		  if(self::get_valueI('sizeOpt') == 0){
		  $window_specify = false;
		  $window_max = true;
		  } else {
		  $window_specify = true;
		  $window_max = false;
		  }


		  $window_max_code = we_html_forms::radiobutton('0', self::get_valueI('sizeOpt') == 0, 'newconf[sizeOpt]', g_l('prefs', '[maximize]'), true, '', "document.getElementsByName('newconf[weWidth]')[0].disabled = true;document.getElementsByName('newconf[weHeight]')[0].disabled = true;");


		  $window_specify_code = we_html_forms::radiobutton('1', !(self::get_valueI('sizeOpt') == 0), 'newconf[sizeOpt]', g_l('prefs', '[specify]'), true, '', "document.getElementsByName('newconf[weWidth]')[0].disabled = false;document.getElementsByName('newconf[weHeight]')[0].disabled = false;");


		  $window_specify_table = (new we_html_table(['class' => 'default', 'style' => 'margin-top:10px;margin-left:50px;'], 1, 3))
		  ->setCol(0, 0, we_html_tools::textInput('newconf[weWidth]', (self::get_valueI('sizeOpt') ? self::get_valueS('weWidth') : ''), 4, [
		  'disabled' => (self::get_valueI('sizeOpt') == 0 ? 'disabled' : '')], "number", '60px'))
		  ->setCol(0, 1, ['style' => 'padding:0 .5em;vertical-align: middle;'], 'x')
		  ->setCol(0, 2, we_html_tools::textInput('newconf[weHeight]', (self::get_valueI('sizeOpt') ? self::get_valueS('weHeight') : ''), 4, [
		  'disabled' => (self::get_valueI('sizeOpt') == 0 ? 'disabled' : "")], "number", '60px'));


		  $window_current_dimension_table = we_html_button::create_button('apply_current_dimension', "javascript:document.getElementsByName('newconf[sizeOpt]')[1].checked = true;document.getElementsByName('newconf[weWidth]')[0].disabled = false;document.getElementsByName('newconf[weHeight]')[0].disabled = false;document.getElementsByName('newconf[weWidth]')[0].value = parent.opener.top.window.outerWidth;document.getElementsByName('newconf[weHeight]')[0].value = parent.opener.top.window.outerHeight;");


		  $window_html = (new we_html_table(['class' => 'default withSpace'], 3, 1))
		  ->setCol(0, 0, $window_max_code)
		  ->setCol(1, 0, $window_specify_code . $window_specify_table->getHtml())
		  ->setCol(2, 0, ['style' => 'padding-left:50px;'], $window_current_dimension_table);


		  $settings->add((new we_html_multiIconBoxContentLine(g_l('prefs', '[dimension]'), 'html' => $window_html->getHtml(),
		  ->space(we_html_multiIconBox::SPACE_BIG)]);
		 */
		return self::create_dialog('', $settings, -1);
	}

	/**
	 * This builds every single dialog (of a tab).
	 *
	 * @param          $selected_setting                       string              (optional)
	 *
	 * @see            render_dialog()
	 *
	 * @return         string
	 */
	private static function build_dialog(string $selected_setting = 'ui'): string{
		switch($selected_setting){
			case 'save':
				return self::create_dialog('', (new we_html_multiIconBoxContent())->add(new we_html_multiIconBoxContentLine('', g_l('prefs', '[save]'))));
			case 'saved'://SAVED SUCCESSFULLY DIALOG
				return self::create_dialog('', (new we_html_multiIconBoxContent())->add(new we_html_multiIconBoxContentLine('', g_l('prefs', '[saved]'))));
			case 'ui':
				return self::getUITab();
			case 'defaultAttribs':
				return self::getDefaultSettingsTab();
			case 'cache':
				return self::getCacheTab();
			case 'countries':
				return self::getCountryTab();
			case 'language':
				return self::getLanguagesTab();
			case 'editor':
				return self::getEditorTab();
			case 'recipients':
				return self::getReceipientsTab();
			case 'modules':
				return self::getModulesTab();
			case 'proxy':
				return self::getProxyTab();
			case 'advanced':
				return self::getAdvancedTab();
			case 'system':
				return self::getSystemTab();
			case 'seolinks':
				return self::getSeoDialog();
			case 'error_handling':
				return self::getErrorTab();
			case 'security':
				return self::getSecurityTab();
			case 'email':
				return self::getEmailTab();
			case 'versions':
				return self::getVersionTab();
		}

		return 'No rights.';
	}

	/**
	 * This functions renders the complete dialog.
	 *
	 * @return         string
	 */
	private static function render_dialog(): string{
		$tabname = we_base_request::string('tabname', 'setting_ui');

		// Check configuration file for all needed variables => since included in startup, nothing should change
		we_base_preferences::check_global_config();
		$tabs = array_keys(self::TABS);
		$tabs[] = 'save';
		$ret = '';

		foreach($tabs as $tab){
			$ret .= we_html_element::div(['id' => 'setting_' . $tab, 'style' => ($tabname === 'setting_' . $tab ? '' : 'display: none;')], self::build_dialog($tab));
		}
		return $ret;
	}

	/**
	 *
	 * @return array{dosave:bool,acerr:bool,acmsg:string}
	 */
	private static function saveSettings(): array{
		$acError = false;
		$acErrorMsg = '';

		// check seemode start document | object
		switch(we_base_request::string('seem_start_type')){
			case 'document':
				$doc = we_base_request::int('seem_start_document', 0);
				if(!$doc){
					$acError = true;
					$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
				} else {
					$folder = fB('SELECT IsFolder FROM ' . FILE_TABLE . ' WHERE ID=' . $doc);
					if($folder){
						$acError = true;
						$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
					}
				}
				break;
			/* case 'weapp':
			  $app = we_base_request::string('newconf', '', 'seem_start_weapp');
			  if(!$app){
			  $acError = true;
			  $acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
			  }
			  break; */
			case 'object':
				$obj = we_base_request::int('seem_start_object', 0);
				if(!$obj){
					$acError = true;
					$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
				} else if(fB('SELECT IsFolder FROM ' . OBJECT_FILES_TABLE . ' WHERE ID=' . $obj)){
					$acError = true;
					$acErrorMsg = sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[seem_startdocument]'), g_l('prefs', '[tab][ui]')) . "\\n";
				}
				break;
		}
		// check sidebar document
		if(!we_base_request::bool('newconf', false, 'SIDEBAR_DISABLED') && we_base_request::_(we_base_request::FILE, 'ui_sidebar_file_name')){
			if(fB('SELECT IsFolder FROM ' . FILE_TABLE . ' WHERE ID=' . we_base_request::int('newconf', 0, 'SIDEBAR_DEFAULT_DOCUMENT'))){
				$acError = true;
				$acErrorMsg .= sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[sidebar]') . ' / ' . g_l('prefs', '[sidebar_document]'), g_l('prefs', '[tab][ui]')) . "\\n";
			}
		}
		// check doc for error on none existing objects
		if(we_base_request::_(we_base_request::FILE, 'error_document_no_objectfile_text')){
			if(fB('SELECT IsFolder FROM ' . FILE_TABLE . ' WHERE ID=' . we_base_request::int('newconf', 0, 'ERROR_DOCUMENT_NO_OBJECTFILE'))){
				$acError = true;
				$acErrorMsg .= sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[error_no_object_found]'), g_l('prefs', '[tab][error_handling]')) . "\\n";
			}
		}
		// check if versioning number is correct
		if(($cnt = we_base_request::int('newconf', 0, 'VERSIONS_ANZAHL'))){
			if(!(abs($cnt) == $cnt && $cnt > 0)){
				$acError = true;
				$acErrorMsg .= sprintf(g_l('alert', '[field_in_tab_notvalid]'), g_l('prefs', '[versioning_anzahl]'), g_l('prefs', '[tab][versions]')) . "\\n";
			}
		}
		return ['dosave' => true, 'acerr' => $acError, 'acmsg' => $acErrorMsg];
	}

	public static function show(): void{
		self::$jsCmd = new we_base_jsCmd();
// Check if we need to save settings
		['dosave' => $doSave, 'acerr' => $acError, 'acmsg' => $acErrorMsg] = (we_base_request::bool('save_settings') ?
			self::saveSettings() :
			['dosave' => false, 'acerr' => false, 'acmsg' => '']);


		if($doSave && !$acError){
			self::save_all_values();
			self::$jsCmd->addMsg(g_l('prefs', '[saved]'), we_base_util::WE_MESSAGE_NOTICE);
			//	self::$jsCmd->addMsg(g_l('prefs', '[error_mail_not_saved]'), we_base_util::WE_MESSAGE_ERROR);

			self::$jsCmd->addCmd('updatePrefs');
			$body = we_html_element::body(['class' => 'weDialogBody'], self::build_dialog('saved')) . '</html>';
		} else {
			$form = we_html_element::form(['onSubmit' => 'return false;', 'name' => we_base_constants::WE_FORM, 'method' => 'post', 'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_gui_editorPreferences'], we_html_element::hidden('save_settings', 0) . self::render_dialog());

			if($acError){
				self::$jsCmd->addMsg(g_l('alert', '[field_in_tab_notvalid_pre]') . "\\n\\n" . $acErrorMsg . "\\n" . g_l('alert', '[field_in_tab_notvalid_post]'), we_base_util::WE_MESSAGE_ERROR);
			}

			$body = we_html_element::body(['class' => 'weDialogBody', 'onload' => 'startPrefs();'], $form);
		}


		echo we_html_tools::getHtmlTop('', we_html_element::jsScript(JS_DIR . 'preferences.js') . self::$jsCmd->getCmds(), $body);
	}

}
